@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -254,6 +254,15 @@ static inline void key_generic_save(void)
 	wins_update(FLAG_STA);
 }
 
+static inline void key_generic_reload(void)
+{
+	io_reload_data();
+	do_storage(0);
+	notify_check_next_app(1);
+	ui_calendar_monthly_view_cache_set_invalid();
+	wins_update(FLAG_ALL);
+}
+
 static inline void key_generic_import(void)
 {
 	wins_erase_status_bar();
@@ -439,6 +448,7 @@ static inline void key_generic_cmd(void)
 	char cmd[BUFSIZ] = "";
 	char *cmd_name;
 	int valid = 0, force = 0;
+	char *error_msg;
 
 	status_mesg(_("Command:"), "");
 	if (getstring(win[STA].p, cmd, BUFSIZ, 0, 1) != GETSTRING_VALID)
@@ -469,26 +479,19 @@ static inline void key_generic_cmd(void)
 		char *topic = strtok(NULL, " ");
 
 		if (!display_help(topic)) {
-			char error_msg[BUFSIZ];
-
-			snprintf(error_msg, BUFSIZ,
-					_("Help topic does not exist: %s"),
-					topic);
-			error_msg[BUFSIZ - 1] = '\0';
-
+			asprintf(&error_msg,
+				 _("Help topic does not exist: %s"), topic);
 			warnbox(error_msg);
+			mem_free(error_msg);
 		}
 
 		valid = 1;
 	}
 
 	if (!valid) {
-		char error_msg[BUFSIZ];
-
-		snprintf(error_msg, BUFSIZ, _("No such command: %s"), cmd);
-		error_msg[BUFSIZ - 1] = '\0';
-
+		asprintf(&error_msg, _("No such command: %s"), cmd);
 		warnbox(error_msg);
+		mem_free(error_msg);
 	}
 
 cleanup:
@@ -518,6 +521,7 @@ int main(int argc, char **argv)
 
 	/* Initialize non-thread-safe data structures. */
 	event_llist_init();
+	recur_event_llist_init();
 	todo_init_list();
 
 	/*
@@ -587,8 +591,9 @@ int main(int argc, char **argv)
 	config_load();
 	wins_erase_status_bar();
 	io_load_keys(conf.pager);
-	io_load_todo();
-	io_load_app();
+	io_load_data(NULL);
+	io_unset_modified();
+	wins_slctd_set(conf.default_panel);
 	wins_resize();
 	/*
 	 * Refresh the hidden key handler window here to prevent wgetch() from
@@ -602,7 +607,6 @@ int main(int argc, char **argv)
 	do_storage(1);
 	ui_todo_load_items();
 	ui_todo_sel_reset();
-	wins_slctd_set(conf.default_panel);
 	wins_update(FLAG_ALL);
 
 	/* Start miscellaneous threads. */
@@ -621,6 +625,11 @@ int main(int argc, char **argv)
 			wins_reset();
 		}
 
+		if (want_reload) {
+			want_reload = 0;
+			key_generic_reload();
+		}
+
 		key = keys_getch(win[KEY].p, &count, &reg);
 		switch (key) {
 		case KEY_GENERIC_REDRAW:
@@ -649,6 +658,7 @@ int main(int argc, char **argv)
 			HANDLE_KEY(KEY_VIEW_NOTE, key_view_note);
 			HANDLE_KEY(KEY_GENERIC_HELP, key_generic_help);
 			HANDLE_KEY(KEY_GENERIC_SAVE, key_generic_save);
+			HANDLE_KEY(KEY_GENERIC_RELOAD, key_generic_reload);
 			HANDLE_KEY(KEY_GENERIC_IMPORT, key_generic_import);
 			HANDLE_KEY(KEY_GENERIC_EXPORT, key_generic_export);
 			HANDLE_KEY(KEY_GENERIC_PREV_DAY, key_generic_prev_day);
