@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -103,8 +103,9 @@
 #define DPID_PATH        DIR_NAME DPID_PATH_NAME
 #define NOTES_DIR        DIR_NAME NOTES_DIR_NAME
 
-#define DEFAULT_EDITOR   "vi"
-#define DEFAULT_PAGER    "less"
+#define DEFAULT_EDITOR     "vi"
+#define DEFAULT_PAGER      "less"
+#define DEFAULT_MERGETOOL  "vimdiff"
 
 #define ATTR_FALSE    0
 #define ATTR_TRUE     1
@@ -116,7 +117,6 @@
 
 #define STATUSHEIGHT  2
 #define MAX_NOTESIZ   40
-#define TMPEXTSIZ     6
 
 /* Format for appointment hours is: HH:MM */
 #define HRMIN_SIZE 6
@@ -259,6 +259,7 @@ struct conf {
 	unsigned progress_bar;
 	const char *editor;
 	const char *pager;
+	const char *mergetool;
 	char output_datefmt[BUFSIZ];	/* format for displaying date */
 	int input_datefmt;	/* format for reading date */
 };
@@ -372,7 +373,7 @@ union aptev_ptr {
 	struct recur_event *rev;
 };
 
-/* Available types of items. */
+/* Available item types in the calendar view. */
 enum day_item_type {
 	DAY_HEADING = 1,
 	RECUR_EVNT,
@@ -382,6 +383,38 @@ enum day_item_type {
 	APPT
 };
 
+/* Available item types. */
+enum item_type {
+	TYPE_EVNT,
+	TYPE_APPT,
+	TYPE_RECUR_EVNT,
+	TYPE_RECUR_APPT,
+	TYPE_TODO
+};
+
+/* Available item type masks. */
+#define TYPE_MASK_EVNT (1 << TYPE_EVNT)
+#define TYPE_MASK_APPT (1 << TYPE_APPT)
+#define TYPE_MASK_RECUR_EVNT (1 << TYPE_RECUR_EVNT)
+#define TYPE_MASK_RECUR_APPT (1 << TYPE_RECUR_APPT)
+#define TYPE_MASK_RECUR (TYPE_MASK_RECUR_EVNT | TYPE_MASK_RECUR_APPT)
+#define TYPE_MASK_CAL (TYPE_MASK_EVNT | TYPE_MASK_APPT | TYPE_MASK_RECUR)
+#define TYPE_MASK_TODO (1 << TYPE_TODO)
+#define TYPE_MASK_ALL (TYPE_MASK_CAL | TYPE_MASK_TODO)
+
+/* Filter settings. */
+struct item_filter {
+	int type_mask;
+	regex_t *regex;
+	long start_from;
+	long start_to;
+	long end_from;
+	long end_to;
+	int priority;
+	int completed;
+	int uncompleted;
+};
+
 /* Generic item description (to hold appointments, events...). */
 struct day_item {
 	enum day_item_type type;
@@ -417,6 +450,7 @@ enum key {
 	KEY_GENERIC_HELP,
 	KEY_GENERIC_QUIT,
 	KEY_GENERIC_SAVE,
+	KEY_GENERIC_RELOAD,
 	KEY_GENERIC_COPY,
 	KEY_GENERIC_PASTE,
 	KEY_GENERIC_CHANGE_VIEW,
@@ -460,13 +494,15 @@ enum key {
 	KEY_LOWER_PRIORITY,
 
 	NBKEYS,
-	KEY_UNDEF
-};
+	KEY_UNDEF,
 
-/* To describe a key binding. */
-struct binding {
-	char *label;
-	enum key action;
+	/* Non-configurable, context sensitive key bindings. */
+	KEY_CONFIGMENU_GENERAL,
+	KEY_CONFIGMENU_LAYOUT,
+	KEY_CONFIGMENU_SIDEBAR,
+	KEY_CONFIGMENU_COLOR,
+	KEY_CONFIGMENU_NOTIFY,
+	KEY_CONFIGMENU_KEYS
 };
 
 #define FLAG_CAL (1 << CAL)
@@ -639,7 +675,8 @@ struct apoint *apoint_new(char *, char *, long, long, char);
 unsigned apoint_inday(struct apoint *, long *);
 void apoint_sec2str(struct apoint *, long, char *, char *);
 void apoint_write(struct apoint *, FILE *);
-struct apoint *apoint_scan(FILE *, struct tm, struct tm, char, char *);
+struct apoint *apoint_scan(FILE *, struct tm, struct tm, char, char *,
+			   struct item_filter *);
 void apoint_delete(struct apoint *);
 struct notify_app *apoint_check_next(struct notify_app *, long);
 void apoint_switch_notify(struct apoint *);
@@ -681,7 +718,6 @@ unsigned config_save(void);
 void custom_init_attr(void);
 void custom_apply_attr(WINDOW *, int);
 void custom_remove_attr(WINDOW *, int);
-void custom_config_bar(void);
 void custom_layout_config(void);
 void custom_sidebar_config(void);
 void custom_color_config(void);
@@ -701,7 +737,7 @@ long day_item_get_duration(struct day_item *);
 int day_item_get_state(struct day_item *);
 void day_item_add_exc(struct day_item *, long);
 void day_item_fork(struct day_item *, struct day_item *);
-void day_store_items(long, regex_t *, int);
+void day_store_items(long, int);
 void day_process_storage(struct date *, unsigned);
 void day_display_item_date(struct day_item *, WINDOW *, int, long, int, int);
 void day_display_item(struct day_item *, WINDOW *, int, int, int, int);
@@ -732,7 +768,7 @@ void event_llist_free(void);
 struct event *event_new(char *, char *, long, int);
 unsigned event_inday(struct event *, long *);
 void event_write(struct event *, FILE *);
-struct event *event_scan(FILE *, struct tm, int, char *);
+struct event *event_scan(FILE *, struct tm, int, char *, struct item_filter *);
 void event_delete(struct event *);
 void event_paste_item(struct event *, long);
 
@@ -752,12 +788,16 @@ void ical_export_data(FILE *);
 unsigned io_fprintln(const char *, const char *, ...);
 void io_init(const char *, const char *);
 void io_extract_data(char *, const char *, int);
-unsigned io_save_apts(void);
-unsigned io_save_todo(void);
+void io_save_mutex_lock(void);
+void io_save_mutex_unlock(void);
+unsigned io_save_apts(const char *);
+unsigned io_save_todo(const char *);
 unsigned io_save_keys(void);
 void io_save_cal(enum save_display);
-void io_load_app(void);
-void io_load_todo(void);
+void io_load_app(struct item_filter *);
+void io_load_todo(struct item_filter *);
+void io_load_data(struct item_filter *);
+void io_reload_data(void);
 void io_load_keys(const char *);
 int io_check_dir(const char *);
 unsigned io_dir_exists(const char *);
@@ -776,8 +816,12 @@ void io_stop_psave_thread(void);
 void io_set_lock(void);
 unsigned io_dump_pid(char *);
 unsigned io_get_pid(char *);
+int io_files_equal(const char *, const char *);
 int io_file_is_empty(char *);
 int io_file_cp(const char *, const char *);
+void io_unset_modified(void);
+void io_set_modified(void);
+int io_get_modified(void);
 
 /* keys.c */
 void keys_init(void);
@@ -794,8 +838,7 @@ int keys_action_count_keys(enum key);
 const char *keys_action_firstkey(enum key);
 const char *keys_action_nkey(enum key, int);
 char *keys_action_allkeys(enum key);
-void keys_display_bindings_bar(WINDOW *, struct binding *[], int, int,
-			       int, struct binding *);
+void keys_display_bindings_bar(WINDOW *, int *, int, int, int);
 void keys_popup_info(enum key);
 void keys_save_bindings(FILE *);
 int keys_check_missing_bindings(void);
@@ -892,6 +935,7 @@ void recur_apoint_free_bkp(void);
 void recur_event_free(struct recur_event *);
 void recur_apoint_free(struct recur_apoint *);
 void recur_apoint_llist_init(void);
+void recur_event_llist_init(void);
 void recur_apoint_llist_free(void);
 void recur_event_llist_free(void);
 struct recur_apoint *recur_apoint_new(char *, char *, long, long, char,
@@ -902,9 +946,10 @@ char recur_def2char(enum recur_type);
 int recur_char2def(char);
 struct recur_apoint *recur_apoint_scan(FILE *, struct tm, struct tm,
 				       char, int, struct tm, char *,
-				       llist_t *, char);
+				       llist_t *, char, struct item_filter *);
 struct recur_event *recur_event_scan(FILE *, struct tm, int, char,
-				     int, struct tm, char *, llist_t *);
+				     int, struct tm, char *, llist_t *,
+				     struct item_filter *);
 void recur_apoint_write(struct recur_apoint *, FILE *);
 void recur_event_write(struct recur_event *, FILE *);
 void recur_save_data(FILE *);
@@ -1022,7 +1067,7 @@ long now(void);
 char *nowstr(void);
 void print_bool_option_incolor(WINDOW *, unsigned, int, int);
 const char *get_tempdir(void);
-char *new_tempfile(const char *, int);
+char *new_tempfile(const char *);
 int check_date(unsigned, unsigned, unsigned);
 int parse_date(const char *, enum datefmt, int *, int *, int *,
 	       struct date *);
@@ -1041,6 +1086,8 @@ void print_recur_apoint(const char *, long, unsigned,
 			struct recur_apoint *);
 void print_recur_event(const char *, long, struct recur_event *);
 void print_todo(const char *, struct todo *);
+int vasprintf(char **, const char *, va_list);
+int asprintf(char **, const char *, ...);
 
 /* vars.c */
 extern int col, row;
@@ -1049,6 +1096,7 @@ extern unsigned colorize;
 extern int foreground, background;
 extern enum ui_mode ui_mode;
 extern int read_only;
+extern int want_reload;
 extern const char *datefmt_str[DATE_FORMATS];
 extern int days[12];
 extern const char *monthnames[12];
@@ -1115,7 +1163,9 @@ void wins_reset_noupdate(void);
 void wins_reset(void);
 void wins_prepare_external(void);
 void wins_unprepare_external(void);
-void wins_launch_external(const char *, const char *);
+void wins_launch_external(const char *[]);
+void wins_set_bindings(int *, int);
+void wins_update_bindings(void);
 void wins_status_bar(void);
 void wins_erase_status_bar(void);
 void wins_other_status_page(int);
