@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,7 +40,7 @@
 #include "calcurse.h"
 
 typedef int (*config_fn_parse_t) (void *, const char *);
-typedef int (*config_fn_serialize_t) (char *, void *);
+typedef int (*config_fn_serialize_t) (char **, void *);
 
 struct confvar {
 	const char *key;
@@ -50,29 +50,29 @@ struct confvar {
 };
 
 static int config_parse_bool(unsigned *, const char *);
-static int config_serialize_bool(char *, unsigned *);
+static int config_serialize_bool(char **, unsigned *);
 static int config_parse_int(int *, const char *);
-static int config_serialize_int(char *, int *);
+static int config_serialize_int(char **, int *);
 static int config_parse_unsigned(unsigned *, const char *);
-static int config_serialize_unsigned(char *, unsigned *);
+static int config_serialize_unsigned(char **, unsigned *);
 static int config_parse_str(char *, const char *);
-static int config_serialize_str(char *, const char *);
+static int config_serialize_str(char **, const char *);
 static int config_parse_calendar_view(void *, const char *);
-static int config_serialize_calendar_view(char *, void *);
+static int config_serialize_calendar_view(char **, void *);
 static int config_parse_default_panel(void *, const char *);
-static int config_serialize_default_panel(char *, void *);
+static int config_serialize_default_panel(char **, void *);
 static int config_parse_first_day_of_week(void *, const char *);
-static int config_serialize_first_day_of_week(char *, void *);
+static int config_serialize_first_day_of_week(char **, void *);
 static int config_parse_color_theme(void *, const char *);
-static int config_serialize_color_theme(char *, void *);
+static int config_serialize_color_theme(char **, void *);
 static int config_parse_layout(void *, const char *);
-static int config_serialize_layout(char *, void *);
+static int config_serialize_layout(char **, void *);
 static int config_parse_sidebar_width(void *, const char *);
-static int config_serialize_sidebar_width(char *, void *);
+static int config_serialize_sidebar_width(char **, void *);
 static int config_parse_output_datefmt(void *, const char *);
-static int config_serialize_output_datefmt(char *, void *);
+static int config_serialize_output_datefmt(char **, void *);
 static int config_parse_input_datefmt(void *, const char *);
-static int config_serialize_input_datefmt(char *, void *);
+static int config_serialize_input_datefmt(char **, void *);
 
 #define CONFIG_HANDLER_BOOL(var) (config_fn_parse_t) config_parse_bool, \
   (config_fn_serialize_t) config_serialize_bool, &(var)
@@ -293,37 +293,27 @@ static int config_set_conf(const char *key, const char *value)
 	return -1;
 }
 
-static int config_serialize_bool(char *dest, unsigned *val)
+static int config_serialize_bool(char **dest, unsigned *val)
 {
-	if (*val) {
-		dest[0] = 'y';
-		dest[1] = 'e';
-		dest[2] = 's';
-		dest[3] = '\0';
-	} else {
-		dest[0] = 'n';
-		dest[1] = 'o';
-		dest[2] = '\0';
-	}
-
+	*dest = mem_strdup(*val ? "yes" : "no");
 	return 1;
 }
 
-static int config_serialize_unsigned(char *dest, unsigned *val)
+static int config_serialize_unsigned(char **dest, unsigned *val)
 {
-	snprintf(dest, BUFSIZ, "%u", *val);
+	asprintf(dest, "%u", *val);
 	return 1;
 }
 
-static int config_serialize_int(char *dest, int *val)
+static int config_serialize_int(char **dest, int *val)
 {
-	snprintf(dest, BUFSIZ, "%d", *val);
+	asprintf(dest, "%d", *val);
 	return 1;
 }
 
-static int config_serialize_str(char *dest, const char *val)
+static int config_serialize_str(char **dest, const char *val)
 {
-	strncpy(dest, val, BUFSIZ);
+	*dest = mem_strdup(val);
 	return 1;
 }
 
@@ -335,13 +325,14 @@ static int config_serialize_str(char *dest, const char *val)
  * If ncurses library was compiled with --enable-ext-funcs,
  * then default color is -1.
  */
-static void config_color_theme_name(char *theme_name)
+static char *config_color_theme_name(void)
 {
 #define MAXCOLORS		8
 #define NBCOLORS		2
 #define DEFAULTCOLOR		255
 #define DEFAULTCOLOR_EXT	-1
 
+	char *theme;
 	int i;
 	short color[NBCOLORS];
 	const char *color_name[NBCOLORS];
@@ -358,88 +349,88 @@ static void config_color_theme_name(char *theme_name)
 	};
 
 	if (!colorize) {
-		strncpy(theme_name, "none", BUFSIZ);
-	} else {
-		pair_content(COLR_CUSTOM, &color[0], &color[1]);
-		for (i = 0; i < NBCOLORS; i++) {
-			if ((color[i] == DEFAULTCOLOR)
-			    || (color[i] == DEFAULTCOLOR_EXT)) {
-				color_name[i] = default_color;
-			} else if (color[i] >= 0 && color[i] <= MAXCOLORS) {
-				color_name[i] = name[color[i]];
-			} else {
-				EXIT(_("unknown color"));
-				/* NOTREACHED */
-			}
-		}
-		snprintf(theme_name, BUFSIZ, "%s on %s", color_name[0],
-			 color_name[1]);
+		return mem_strdup("none");
 	}
+
+	pair_content(COLR_CUSTOM, &color[0], &color[1]);
+	for (i = 0; i < NBCOLORS; i++) {
+		if ((color[i] == DEFAULTCOLOR)
+		    || (color[i] == DEFAULTCOLOR_EXT)) {
+			color_name[i] = default_color;
+		} else if (color[i] >= 0 && color[i] <= MAXCOLORS) {
+			color_name[i] = name[color[i]];
+		} else {
+			EXIT(_("unknown color"));
+			/* NOTREACHED */
+		}
+	}
+	asprintf(&theme, "%s on %s", color_name[0], color_name[1]);
+	return theme;
 }
 
-static int config_serialize_calendar_view(char *buf, void *dummy)
+static int config_serialize_calendar_view(char **buf, void *dummy)
 {
 	if (ui_calendar_get_view() == CAL_WEEK_VIEW)
-		strcpy(buf, "weekly");
+		*buf = mem_strdup("weekly");
 	else
-		strcpy(buf, "monthly");
+		*buf = mem_strdup("monthly");
 
 	return 1;
 }
 
-static int config_serialize_default_panel(char *buf, void *dummy)
+static int config_serialize_default_panel(char **buf, void *dummy)
 {
 	if (conf.default_panel == CAL)
-		strcpy(buf, "calendar");
+		*buf = mem_strdup("calendar");
 	else if (conf.default_panel == APP)
-		strcpy(buf, "appointments");
+		*buf = mem_strdup("appointments");
 	else
-		strcpy(buf, "todo");
+		*buf = mem_strdup("todo");
 
 	return 1;
 }
 
-static int config_serialize_first_day_of_week(char *buf, void *dummy)
+static int config_serialize_first_day_of_week(char **buf, void *dummy)
 {
 	if (ui_calendar_week_begins_on_monday())
-		strcpy(buf, "monday");
+		*buf = mem_strdup("monday");
 	else
-		strcpy(buf, "sunday");
+		*buf = mem_strdup("sunday");
 
 	return 1;
 }
 
-static int config_serialize_color_theme(char *buf, void *dummy)
+static int config_serialize_color_theme(char **buf, void *dummy)
 {
-	config_color_theme_name(buf);
+	*buf = config_color_theme_name();
 	return 1;
 }
 
-static int config_serialize_layout(char *buf, void *dummy)
+static int config_serialize_layout(char **buf, void *dummy)
 {
 	int tmp = wins_layout();
 	return config_serialize_int(buf, &tmp);
 }
 
-static int config_serialize_sidebar_width(char *buf, void *dummy)
+static int config_serialize_sidebar_width(char **buf, void *dummy)
 {
 	int tmp = wins_sbar_wperc();
 	return config_serialize_int(buf, &tmp);
 }
 
-static int config_serialize_output_datefmt(char *buf, void *dummy)
+static int config_serialize_output_datefmt(char **buf, void *dummy)
 {
 	return config_serialize_str(buf, conf.output_datefmt);
 }
 
-static int config_serialize_input_datefmt(char *buf, void *dummy)
+static int config_serialize_input_datefmt(char **buf, void *dummy)
 {
 	return config_serialize_int(buf, &conf.input_datefmt);
 }
 
 /* Serialize the value of a configuration variable. */
 static int
-config_serialize_conf(char *buf, const char *key,
+config_serialize_conf(char **buf, const char *key,
 		      struct config_save_status *status)
 {
 	int i;
@@ -449,8 +440,7 @@ config_serialize_conf(char *buf, const char *key,
 
 	for (i = 0; i < ARRAY_SIZE(confmap); i++) {
 		if (!strcmp(confmap[i].key, key)) {
-			if (confmap[i].
-			    fn_serialize(buf, confmap[i].target)) {
+			if (confmap[i].fn_serialize(buf, confmap[i].target)) {
 				if (status)
 					status->done[i] = 1;
 				return 1;
@@ -480,7 +470,7 @@ config_file_walk(config_fn_walk_cb_t fn_cb,
 			break;
 		io_extract_data(e_conf, buf, sizeof buf);
 
-		if (*e_conf == '\0') {
+		if (*e_conf == '\0' || *e_conf == '#') {
 			if (fn_junk_cb)
 				fn_junk_cb(buf, data);
 			continue;
@@ -527,6 +517,8 @@ config_file_walk(config_fn_walk_cb_t fn_cb,
 			if (fgets(buf, sizeof buf, data_file) == NULL)
 				break;
 			io_extract_data(e_conf, buf, sizeof buf);
+			if (*e_conf == '#')
+				*e_conf = '\0';
 			value = e_conf;
 		}
 
@@ -560,9 +552,9 @@ void config_load(void)
 
 static int config_save_cb(const char *key, const char *value, void *status)
 {
-	char buf[BUFSIZ];
+	char *buf;
 	int result =
-	    config_serialize_conf(buf, key,
+	    config_serialize_conf(&buf, key,
 				  (struct config_save_status *)status);
 
 	if (result < 0) {
@@ -579,6 +571,7 @@ static int config_save_cb(const char *key, const char *value, void *status)
 	fputs(buf, ((struct config_save_status *)status)->fp);
 	fputc('\n', ((struct config_save_status *)status)->fp);
 
+	mem_free(buf);
 	return 1;
 }
 
@@ -591,25 +584,21 @@ static int config_save_junk_cb(const char *data, void *status)
 /* Save the user configuration. */
 unsigned config_save(void)
 {
-	char tmppath[BUFSIZ];
-	char *tmpext;
+	char *tmpprefix = NULL, *tmppath = NULL;
 	struct config_save_status status;
 	int i;
+	int ret = 0;
 
 	if (read_only)
 		return 1;
 
-	strncpy(tmppath, get_tempdir(), BUFSIZ);
-	tmppath[BUFSIZ - 1] = '\0';
-	strncat(tmppath, "/" CONF_PATH_NAME ".", BUFSIZ - strlen(tmppath) - 1);
-	if ((tmpext = new_tempfile(tmppath, TMPEXTSIZ)) == NULL)
-		return 0;
-	strncat(tmppath, tmpext, BUFSIZ - strlen(tmppath) - 1);
-	mem_free(tmpext);
+	asprintf(&tmpprefix, "%s/%s", get_tempdir(), CONF_PATH_NAME);
+	if ((tmppath = new_tempfile(tmpprefix)) == NULL)
+		goto cleanup;
 
 	status.fp = fopen(tmppath, "w");
 	if (!status.fp)
-		return 0;
+		goto cleanup;
 
 	memset(status.done, 0, sizeof(status.done));
 
@@ -627,5 +616,9 @@ unsigned config_save(void)
 	if (io_file_cp(tmppath, path_conf))
 		unlink(tmppath);
 
-	return 1;
+	ret = 1;
+cleanup:
+	mem_free(tmpprefix);
+	mem_free(tmppath);
+	return ret;
 }
