@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -93,52 +93,16 @@ void custom_remove_attr(WINDOW * win, int attr_num)
 		wattroff(win, attr.nocolor[attr_num]);
 }
 
-/* Draws the configuration bar */
-void custom_config_bar(void)
-{
-	const int SMLSPC = 2;
-	const int SPC = 15;
-
-	custom_apply_attr(win[STA].p, ATTR_HIGHEST);
-	mvwaddstr(win[STA].p, 0, 2, "Q");
-	mvwaddstr(win[STA].p, 1, 2, "G");
-	mvwaddstr(win[STA].p, 0, 2 + SPC, "L");
-	mvwaddstr(win[STA].p, 1, 2 + SPC, "S");
-	mvwaddstr(win[STA].p, 0, 2 + 2 * SPC, "C");
-	mvwaddstr(win[STA].p, 1, 2 + 2 * SPC, "N");
-	mvwaddstr(win[STA].p, 0, 2 + 3 * SPC, "K");
-	custom_remove_attr(win[STA].p, ATTR_HIGHEST);
-
-	mvwaddstr(win[STA].p, 0, 2 + SMLSPC, _("Exit"));
-	mvwaddstr(win[STA].p, 1, 2 + SMLSPC, _("General"));
-	mvwaddstr(win[STA].p, 0, 2 + SPC + SMLSPC, _("Layout"));
-	mvwaddstr(win[STA].p, 1, 2 + SPC + SMLSPC, _("Sidebar"));
-	mvwaddstr(win[STA].p, 0, 2 + 2 * SPC + SMLSPC, _("Color"));
-	mvwaddstr(win[STA].p, 1, 2 + 2 * SPC + SMLSPC, _("Notify"));
-	mvwaddstr(win[STA].p, 0, 2 + 3 * SPC + SMLSPC, _("Keys"));
-
-	wnoutrefresh(win[STA].p);
-	wmove(win[STA].p, 0, 0);
-	wins_doupdate();
-}
-
 static void layout_selection_bar(void)
 {
-	struct binding quit = { _("Exit"), KEY_GENERIC_QUIT };
-	struct binding select = { _("Select"), KEY_GENERIC_SELECT };
-	struct binding up = { _("Up"), KEY_MOVE_UP };
-	struct binding down = { _("Down"), KEY_MOVE_DOWN };
-	struct binding left = { _("Left"), KEY_MOVE_LEFT };
-	struct binding right = { _("Right"), KEY_MOVE_RIGHT };
-	struct binding help = { _("Help"), KEY_GENERIC_HELP };
-
-	struct binding *bindings[] = {
-		&quit, &select, &up, &down, &left, &right, &help
+	static int bindings[] = {
+		KEY_GENERIC_QUIT, KEY_GENERIC_SELECT, KEY_MOVE_UP,
+		KEY_MOVE_DOWN, KEY_MOVE_LEFT, KEY_MOVE_RIGHT, KEY_GENERIC_HELP
 	};
 	int bindings_size = ARRAY_SIZE(bindings);
 
 	keys_display_bindings_bar(win[STA].p, bindings, bindings_size, 0,
-				  bindings_size, NULL);
+				  bindings_size);
 }
 
 #define NBLAYOUTS     8
@@ -269,19 +233,13 @@ void custom_layout_config(void)
 /* Sidebar configuration screen. */
 void custom_sidebar_config(void)
 {
-	struct binding quit = { _("Exit"), KEY_GENERIC_QUIT };
-	struct binding inc = { _("Width +"), KEY_MOVE_UP };
-	struct binding dec = { _("Width -"), KEY_MOVE_DOWN };
-	struct binding help = { _("Help"), KEY_GENERIC_HELP };
-	struct binding *bindings[] = {
-		&inc, &dec, &help, &quit
+	static int bindings[] = {
+		KEY_GENERIC_QUIT, KEY_MOVE_UP, KEY_MOVE_DOWN, KEY_GENERIC_HELP
 	};
-	int ch, bindings_size;
-
-	bindings_size = ARRAY_SIZE(bindings);
+	int ch, bindings_size = ARRAY_SIZE(bindings);
 
 	keys_display_bindings_bar(win[STA].p, bindings, bindings_size, 0,
-				  bindings_size, NULL);
+				  bindings_size);
 	wins_doupdate();
 
 	while ((ch =
@@ -308,7 +266,7 @@ void custom_sidebar_config(void)
 			wins_update_panels(FLAG_ALL);
 			keys_display_bindings_bar(win[STA].p, bindings,
 						  bindings_size, 0,
-						  bindings_size, NULL);
+						  bindings_size);
 			wins_doupdate();
 		}
 	}
@@ -351,21 +309,14 @@ void custom_confwin_init(struct window *confwin, const char *label)
 
 static void color_selection_bar(void)
 {
-	struct binding quit = { _("Exit"), KEY_GENERIC_QUIT };
-	struct binding select = { _("Select"), KEY_GENERIC_SELECT };
-	struct binding nocolor = { _("No color"), KEY_GENERIC_CANCEL };
-	struct binding up = { _("Up"), KEY_MOVE_UP };
-	struct binding down = { _("Down"), KEY_MOVE_DOWN };
-	struct binding left = { _("Left"), KEY_MOVE_LEFT };
-	struct binding right = { _("Right"), KEY_MOVE_RIGHT };
-
-	struct binding *bindings[] = {
-		&quit, &nocolor, &up, &down, &left, &right, &select
+	static int bindings[] = {
+		KEY_GENERIC_QUIT, KEY_GENERIC_SELECT, KEY_GENERIC_CANCEL,
+		KEY_MOVE_UP, KEY_MOVE_DOWN, KEY_MOVE_LEFT, KEY_GENERIC_SELECT
 	};
 	int bindings_size = ARRAY_SIZE(bindings);
 
 	keys_display_bindings_bar(win[STA].p, bindings, bindings_size, 0,
-				  bindings_size, NULL);
+				  bindings_size);
 }
 
 /*
@@ -567,24 +518,29 @@ void custom_color_config(void)
 	delwin(conf_win.p);
 }
 
+enum {
+	COMPACT_PANELS,
+	DEFAULT_PANEL,
+	AUTO_SAVE,
+	AUTO_GC,
+	PERIODIC_SAVE,
+	CONFIRM_QUIT,
+	CONFIRM_DELETE,
+	SYSTEM_DIAGS,
+	PROGRESS_BAR,
+	FIRST_DAY_OF_WEEK,
+	OUTPUT_DATE_FMT,
+	INPUT_DATE_FMT,
+	NB_OPTIONS
+};
+
 /* Prints the general options. */
 static void print_general_option(int i, WINDOW *win, int y, int hilt, void *cb_data)
 {
-	enum {
-		AUTO_SAVE,
-		AUTO_GC,
-		PERIODIC_SAVE,
-		CONFIRM_QUIT,
-		CONFIRM_DELETE,
-		SYSTEM_DIAGS,
-		PROGRESS_BAR,
-		FIRST_DAY_OF_WEEK,
-		OUTPUT_DATE_FMT,
-		INPUT_DATE_FMT,
-		NB_OPTIONS
-	};
 	const int XPOS = 1;
 	char *opt[NB_OPTIONS] = {
+		"appearance.compactpanels = ",
+		"appearance.defaultpanel = ",
 		"general.autosave = ",
 		"general.autogc = ",
 		"general.periodicsave = ",
@@ -596,25 +552,45 @@ static void print_general_option(int i, WINDOW *win, int y, int hilt, void *cb_d
 		"format.outputdate = ",
 		"format.inputdate = "
 	};
+	const char *panel;
 
 	if (hilt)
 		custom_apply_attr(win, ATTR_HIGHEST);
 	mvwprintw(win, y, XPOS, "%s", opt[i]);
 
 	switch (i) {
-	case 0:
+	case COMPACT_PANELS:
+		print_bool_option_incolor(win, conf.compact_panels, y,
+					  XPOS + strlen(opt[COMPACT_PANELS]));
+		mvwaddstr(win, y + XPOS, 1,
+			  _("(if set to YES, compact panels are used)"));
+		break;
+	case DEFAULT_PANEL:
+		if (conf.default_panel == CAL)
+			panel = _("Calendar");
+		else if (conf.default_panel == APP)
+			panel = _("Appointments");
+		else
+			panel = _("TODO");
+		custom_apply_attr(win, ATTR_HIGHEST);
+		mvwaddstr(win, y, XPOS + strlen(opt[DEFAULT_PANEL]), panel);
+		custom_remove_attr(win, ATTR_HIGHEST);
+		mvwaddstr(win, y + 1, XPOS,
+			  _("(specifies the panel that is selected by default)"));
+		break;
+	case AUTO_SAVE:
 		print_bool_option_incolor(win, conf.auto_save, y,
 					  XPOS + strlen(opt[AUTO_SAVE]));
 		mvwaddstr(win, y + XPOS, 1,
 			  _("(if set to YES, automatic save is done when quitting)"));
 		break;
-	case 1:
+	case AUTO_GC:
 		print_bool_option_incolor(win, conf.auto_gc, y,
 					  XPOS + strlen(opt[AUTO_GC]));
 		mvwaddstr(win, y + 1, XPOS,
 			  _("(run the garbage collector when quitting)"));
 		break;
-	case 2:
+	case PERIODIC_SAVE:
 		custom_apply_attr(win, ATTR_HIGHEST);
 		mvwprintw(win, y, XPOS + strlen(opt[PERIODIC_SAVE]), "%d",
 			  conf.periodic_save);
@@ -623,34 +599,34 @@ static void print_general_option(int i, WINDOW *win, int y, int hilt, void *cb_d
 			  _("(if not null, automatically save data every 'periodic_save' "
 			   "minutes)"));
 		break;
-	case 3:
+	case CONFIRM_QUIT:
 		print_bool_option_incolor(win, conf.confirm_quit, y,
 					  XPOS + strlen(opt[CONFIRM_QUIT]));
 		mvwaddstr(win, y + 1, XPOS,
 			  _("(if set to YES, confirmation is required before quitting)"));
 		break;
-	case 4:
+	case CONFIRM_DELETE:
 		print_bool_option_incolor(win, conf.confirm_delete, y,
 					  XPOS + strlen(opt[CONFIRM_DELETE]));
 		mvwaddstr(win, y + 1, XPOS,
 			  _("(if set to YES, confirmation is required "
 			    "before deleting an event)"));
 		break;
-	case 5:
+	case SYSTEM_DIAGS:
 		print_bool_option_incolor(win, conf.system_dialogs, y,
 					  XPOS + strlen(opt[SYSTEM_DIAGS]));
 		mvwaddstr(win, y + 1, XPOS,
 			  _("(if set to YES, messages about loaded "
 			    "and saved data will be displayed)"));
 		break;
-	case 6:
+	case PROGRESS_BAR:
 		print_bool_option_incolor(win, conf.progress_bar, y,
 					  XPOS + strlen(opt[PROGRESS_BAR]));
 		mvwaddstr(win, y + 1, XPOS,
 			  _("(if set to YES, progress bar will be displayed "
 			    "when saving data)"));
 		break;
-	case 7:
+	case FIRST_DAY_OF_WEEK:
 		custom_apply_attr(win, ATTR_HIGHEST);
 		mvwaddstr(win, y, XPOS + strlen(opt[FIRST_DAY_OF_WEEK]),
 			  ui_calendar_week_begins_on_monday()? _("Monday") :
@@ -659,7 +635,7 @@ static void print_general_option(int i, WINDOW *win, int y, int hilt, void *cb_d
 		mvwaddstr(win, y + 1, XPOS,
 			  _("(specifies the first day of week in the calendar view)"));
 		break;
-	case 8:
+	case OUTPUT_DATE_FMT:
 		custom_apply_attr(win, ATTR_HIGHEST);
 		mvwaddstr(win, y, XPOS + strlen(opt[OUTPUT_DATE_FMT]),
 			  conf.output_datefmt);
@@ -667,7 +643,7 @@ static void print_general_option(int i, WINDOW *win, int y, int hilt, void *cb_d
 		mvwaddstr(win, y + 1, XPOS,
 			  _("(Format of the date to be displayed in non-interactive mode)"));
 		break;
-	case 9:
+	case INPUT_DATE_FMT:
 		custom_apply_attr(win, ATTR_HIGHEST);
 		mvwprintw(win, y, XPOS + strlen(opt[INPUT_DATE_FMT]), "%d",
 			  conf.input_datefmt);
@@ -709,16 +685,29 @@ static void general_option_edit(int i)
 	const char *periodic_save_str =
 	    _("Enter the delay, in minutes, between automatic saves (0 to disable) ");
 	int val;
-	char *buf = malloc(BUFSIZ);
+	char *buf;
+
+	buf = mem_malloc(BUFSIZ);
+	buf[0] = '\0';
 
 	switch (i) {
-	case 0:
+	case COMPACT_PANELS:
+		conf.compact_panels = !conf.compact_panels;
+		resize = 1;
+		break;
+	case DEFAULT_PANEL:
+		if (conf.default_panel == TOD)
+			conf.default_panel = CAL;
+		else
+			conf.default_panel++;
+		break;
+	case AUTO_SAVE:
 		conf.auto_save = !conf.auto_save;
 		break;
-	case 1:
+	case AUTO_GC:
 		conf.auto_gc = !conf.auto_gc;
 		break;
-	case 2:
+	case PERIODIC_SAVE:
 		status_mesg(periodic_save_str, "");
 		if (updatestring(win[STA].p, &buf, 0, 1) == 0) {
 			val = atoi(buf);
@@ -731,22 +720,22 @@ static void general_option_edit(int i)
 		}
 		status_mesg(number_str, keys);
 		break;
-	case 3:
+	case CONFIRM_QUIT:
 		conf.confirm_quit = !conf.confirm_quit;
 		break;
-	case 4:
+	case CONFIRM_DELETE:
 		conf.confirm_delete = !conf.confirm_delete;
 		break;
-	case 5:
+	case SYSTEM_DIAGS:
 		conf.system_dialogs = !conf.system_dialogs;
 		break;
-	case 6:
+	case PROGRESS_BAR:
 		conf.progress_bar = !conf.progress_bar;
 		break;
-	case 7:
+	case FIRST_DAY_OF_WEEK:
 		ui_calendar_change_first_day_of_week();
 		break;
-	case 8:
+	case OUTPUT_DATE_FMT:
 		status_mesg(output_datefmt_str, "");
 		strncpy(buf, conf.output_datefmt,
 			strlen(conf.output_datefmt) + 1);
@@ -756,7 +745,7 @@ static void general_option_edit(int i)
 		}
 		status_mesg(number_str, keys);
 		break;
-	case 9:
+	case INPUT_DATE_FMT:
 		val = status_ask_simplechoice(input_datefmt_prefix,
 					      datefmt_str,
 					      DATE_FORMATS);
@@ -771,6 +760,9 @@ static void general_option_edit(int i)
 /* General configuration. */
 void custom_general_config(void)
 {
+	static int bindings[] = {
+		KEY_GENERIC_QUIT, KEY_MOVE_UP, KEY_MOVE_DOWN, KEY_EDIT_ITEM
+	};
 	struct listbox lb;
 	int ch;
 
@@ -780,8 +772,12 @@ void custom_general_config(void)
 		     general_option_height, print_general_option);
 	listbox_load_items(&lb, 10);
 	listbox_draw_deco(&lb, 0);
-	status_mesg("", "");
 	listbox_display(&lb);
+	wins_set_bindings(bindings, ARRAY_SIZE(bindings));
+	wins_status_bar();
+	wnoutrefresh(win[STA].p);
+	wmove(win[STA].p, 0, 0);
+	wins_doupdate();
 
 	while ((ch = keys_getch(win[KEY].p, NULL, NULL)) != KEY_GENERIC_QUIT) {
 		switch (ch) {
@@ -810,8 +806,11 @@ void custom_general_config(void)
 			}
 		}
 
-		status_mesg("", "");
 		listbox_display(&lb);
+		wins_status_bar();
+		wnoutrefresh(win[STA].p);
+		wmove(win[STA].p, 0, 0);
+		wins_doupdate();
 	}
 
 	listbox_delete(&lb);
@@ -839,14 +838,15 @@ print_keys_bindings(WINDOW * win, int selected_row, int selected_elm,
 
 	noelm = y = 0;
 	for (action = 0; action < NBKEYS; action++) {
-		char actionstr[BUFSIZ];
+		char *actionstr;
 		int nbkeys;
 
 		nbkeys = keys_action_count_keys(action);
-		snprintf(actionstr, BUFSIZ, "%s", keys_get_label(action));
+		asprintf(&actionstr, "%s", keys_get_label(action));
 		if (action == selected_row)
 			custom_apply_attr(win, ATTR_HIGHEST);
 		mvwprintw(win, y, XPOS, "%s ", actionstr);
+		mem_free(actionstr);
 		mvwaddstr(win, y, EQUALPOS, "=");
 		if (nbkeys == 0)
 			mvwaddstr(win, y, KEYPOS, _("undefined"));
@@ -883,22 +883,14 @@ print_keys_bindings(WINDOW * win, int selected_row, int selected_elm,
 
 static void custom_keys_config_bar(void)
 {
-	struct binding quit = { _("Exit"), KEY_GENERIC_QUIT };
-	struct binding info = { _("Key info"), KEY_GENERIC_HELP };
-	struct binding add = { _("Add key"), KEY_ADD_ITEM };
-	struct binding del = { _("Del key"), KEY_DEL_ITEM };
-	struct binding up = { _("Up"), KEY_MOVE_UP };
-	struct binding down = { _("Down"), KEY_MOVE_DOWN };
-	struct binding left = { _("Prev Key"), KEY_MOVE_LEFT };
-	struct binding right = { _("Next Key"), KEY_MOVE_RIGHT };
-
-	struct binding *bindings[] = {
-		&quit, &info, &add, &del, &up, &down, &left, &right
+	static int bindings[] = {
+		KEY_GENERIC_QUIT, KEY_GENERIC_HELP, KEY_ADD_ITEM, KEY_DEL_ITEM,
+		KEY_MOVE_UP, KEY_MOVE_DOWN, KEY_MOVE_LEFT, KEY_MOVE_RIGHT
 	};
 	int bindings_size = ARRAY_SIZE(bindings);
 
 	keys_display_bindings_bar(win[STA].p, bindings, bindings_size, 0,
-				  bindings_size, NULL);
+				  bindings_size);
 }
 
 void custom_keys_config(void)
@@ -1051,13 +1043,27 @@ void custom_keys_config(void)
 
 void custom_config_main(void)
 {
+	static int bindings[] = {
+		KEY_GENERIC_QUIT, KEY_CONFIGMENU_GENERAL,
+		KEY_CONFIGMENU_LAYOUT, KEY_CONFIGMENU_SIDEBAR,
+		KEY_CONFIGMENU_COLOR, KEY_CONFIGMENU_NOTIFY,
+		KEY_CONFIGMENU_KEYS
+	};
 	const char *no_color_support =
 	    _("Sorry, colors are not supported by your terminal\n"
 	      "(Press [ENTER] to continue)");
 	int ch;
 	int old_layout;
 
-	custom_config_bar();
+	wins_set_bindings(bindings, ARRAY_SIZE(bindings));
+	wins_update_border(FLAG_ALL);
+	wins_update_panels(FLAG_ALL);
+	wins_status_bar();
+	if (notify_bar())
+		notify_update_bar();
+	wmove(win[STA].p, 0, 0);
+	wins_doupdate();
+
 	while ((ch = wgetch(win[KEY].p)) != 'q') {
 		switch (ch) {
 		case 'C':
@@ -1067,8 +1073,7 @@ void custom_config_main(void)
 			} else {
 				colorize = 0;
 				wins_erase_status_bar();
-				mvwaddstr(win[STA].p, 0, 0,
-					  no_color_support);
+				mvwaddstr(win[STA].p, 0, 0, no_color_support);
 				wgetch(win[KEY].p);
 			}
 			break;
@@ -1096,10 +1101,16 @@ void custom_config_main(void)
 			custom_sidebar_config();
 			break;
 		default:
-			continue;
+			break;
 		}
-		wins_update(FLAG_ALL);
-		wins_erase_status_bar();
-		custom_config_bar();
+
+		wins_set_bindings(bindings, ARRAY_SIZE(bindings));
+		wins_update_border(FLAG_ALL);
+		wins_update_panels(FLAG_ALL);
+		wins_status_bar();
+		if (notify_bar())
+			notify_update_bar();
+		wmove(win[STA].p, 0, 0);
+		wins_doupdate();
 	}
 }
