@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -163,8 +163,9 @@ void dmon_start(int parent_exit_status)
 	apoint_llist_init();
 	recur_apoint_llist_init();
 	event_llist_init();
+	recur_event_llist_init();
 	todo_init_list();
-	io_load_app();
+	io_load_app(NULL);
 	data_loaded = 1;
 
 	DMON_LOG(_("started at %s\n"), nowstr());
