@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -302,19 +302,32 @@ ical_store_event(char *mesg, char *note, long day, long end,
 		recur_event_new(mesg, note, day, EVENTID, rpt->type,
 				rpt->freq, rpt->until, exc);
 		mem_free(rpt);
-	} else if (end && end != day) {
-		/* Here we have an event that spans over several days. */
-		rpt = mem_malloc(sizeof(ical_rpt_t));
-		rpt->type = RECUR_DAILY;
-		rpt->freq = 1;
-		rpt->count = 0;
-		rpt->until = end;
-		recur_event_new(mesg, note, day, EVENTID, rpt->type,
-				rpt->freq, rpt->until, exc);
-		mem_free(rpt);
-	} else {
-		event_new(mesg, note, day, EVENTID);
+		goto cleanup;
 	}
+
+	if (end == 0 || end - day <= DAYINSEC) {
+		event_new(mesg, note, day, EVENTID);
+		goto cleanup;
+	}
+
+	/*
+	 * Here we have an event that spans over several days.
+	 *
+	 * In iCal, the end specifies when the event is supposed to end, in
+	 * calcurse, the end specifies the time that the last occurrence of the
+	 * event starts, so we need to do some conversion here.
+	 */
+	end = day + ((end - day - 1) / DAYINSEC) * DAYINSEC;
+	rpt = mem_malloc(sizeof(ical_rpt_t));
+	rpt->type = RECUR_DAILY;
+	rpt->freq = 1;
+	rpt->count = 0;
+	rpt->until = end;
+	recur_event_new(mesg, note, day, EVENTID, rpt->type,
+			rpt->freq, rpt->until, exc);
+	mem_free(rpt);
+
+cleanup:
 	mem_free(mesg);
 	erase_note(&note);
 }
@@ -399,8 +412,12 @@ ical_readline_init(FILE * fdi, char *buf, char *lstore, unsigned *ln)
 
 	*buf = *lstore = '\0';
 	if (fgets(lstore, BUFSIZ, fdi)) {
-		if ((eol = strchr(lstore, '\n')) != NULL)
-			*eol = '\0';
+		if ((eol = strchr(lstore, '\n')) != NULL) {
+			if (*(eol - 1) == '\r')
+				*(eol - 1) = '\0';
+			else
+				*eol = '\0';
+		}
 		(*ln)++;
 	}
 }
@@ -413,8 +430,12 @@ static int ical_readline(FILE * fdi, char *buf, char *lstore, unsigned *ln)
 	(*ln)++;
 
 	while (fgets(lstore, BUFSIZ, fdi) != NULL) {
-		if ((eol = strchr(lstore, '\n')) != NULL)
-			*eol = '\0';
+		if ((eol = strchr(lstore, '\n')) != NULL) {
+			if (*(eol - 1) == '\r')
+				*(eol - 1) = '\0';
+			else
+				*eol = '\0';
+		}
 		if (*lstore != SPACE && *lstore != TAB)
 			break;
 		strncat(buf, lstore + 1, BUFSIZ - strlen(buf) - 1);
@@ -488,30 +509,34 @@ static long ical_datetime2long(char *datestr, ical_vevent_e * type)
 
 static long ical_durtime2long(char *timestr)
 {
-	long timelong;
 	char *p;
+	int bytes_read;
+	unsigned hour = 0, min = 0, sec = 0;
 
-	if ((p = strchr(timestr, 'T')) == NULL) {
-		timelong = 0;
-	} else {
-		int nbmatch;
-		struct {
-			unsigned hour, min, sec;
-		} time;
+	if ((p = strchr(timestr, 'T')) == NULL)
+		return 0;
 
-		p++;
-		memset(&time, 0, sizeof time);
-		nbmatch =
-		    sscanf(p, "%uH%uM%uS", &time.hour, &time.min,
-			   &time.sec);
-		if (nbmatch < 1 || nbmatch > 3)
-			timelong = 0;
-		else
-			timelong =
-			    time.hour * HOURINSEC + time.min * MININSEC +
-			    time.sec;
+	p++;
+	if (strchr(p, 'H')) {
+		if (sscanf(p, "%uH%n", &hour, &bytes_read) != 1)
+			return 0;
+		p += bytes_read;
 	}
-	return timelong;
+	if (strchr(p, 'M')) {
+		if (sscanf(p, "%uM%n", &min, &bytes_read) != 1)
+			return 0;
+		p += bytes_read;
+	}
+	if (strchr(p, 'S')) {
+		if (sscanf(p, "%uM%n", &sec, &bytes_read) != 1)
+			return 0;
+		p += bytes_read;
+	}
+
+	if (hour == 0 && min == 0 && sec == 0)
+		return 0;
+
+	return hour * HOURINSEC + min * MININSEC + sec;
 }
 
 /*
@@ -785,7 +810,6 @@ ical_read_exdate(llist_t * exc, FILE * log, char *exstr,
 	char *p, *q;
 	long date;
 
-	LLIST_INIT(exc);
 	if ((p = strchr(exstr, ':')) != NULL) {
 		p++;
 		while ((q = strchr(p, ',')) != NULL) {
@@ -887,6 +911,7 @@ ical_read_event(FILE * fdi, FILE * log, unsigned *noevents,
 
 	vevent_type = UNDEFINED;
 	memset(&vevent, 0, sizeof vevent);
+	LLIST_INIT(&vevent.exc);
 	skip_alarm = 0;
 	while (ical_readline(fdi, buf, lstore, lineno)) {
 		if (skip_alarm) {
