@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -83,6 +83,8 @@ HTABLE_PROTOTYPE(ht_keybindings, ht_keybindings_s)
     HTABLE_GENERATE(ht_keybindings, ht_keybindings_s, load_keys_ht_getkey,
 		load_keys_ht_compare)
 
+static int modified = 0;
+
 /* Draw a progress bar while saving, loading or exporting data. */
 static void progress_bar(progress_bar_t type, int progress)
 {
@@ -164,13 +166,11 @@ static FILE *get_export_stream(enum export_type type)
 	const char *file_ext[IO_EXPORT_NBTYPES] = { "ical", "txt" };
 
 	stream = NULL;
-	stream_name = (char *)mem_malloc(BUFSIZ);
 	if ((home = getenv("HOME")) != NULL)
-		snprintf(stream_name, BUFSIZ, "%s/calcurse.%s", home,
-			 file_ext[type]);
+		asprintf(&stream_name, "%s/calcurse.%s", home, file_ext[type]);
 	else
-		snprintf(stream_name, BUFSIZ, "%s/calcurse.%s",
-			 get_tempdir(), file_ext[type]);
+		asprintf(&stream_name, "%s/calcurse.%s", get_tempdir(),
+			 file_ext[type]);
 
 	while (stream == NULL) {
 		status_mesg(question, "");
@@ -184,8 +184,8 @@ static FILE *get_export_stream(enum export_type type)
 			wgetch(win[KEY].p);
 		}
 	}
-	mem_free(stream_name);
 
+	mem_free(stream_name);
 	return stream;
 }
 
@@ -194,7 +194,7 @@ unsigned io_fprintln(const char *fname, const char *fmt, ...)
 {
 	FILE *fp;
 	va_list ap;
-	char buf[BUFSIZ];
+	char *buf;
 	int ret;
 
 	fp = fopen(fname, "a");
@@ -202,7 +202,7 @@ unsigned io_fprintln(const char *fname, const char *fmt, ...)
 		  strerror(errno));
 
 	va_start(ap, fmt);
-	ret = vsnprintf(buf, sizeof buf, fmt, ap);
+	ret = vasprintf(&buf, fmt, ap);
 	RETVAL_IF(ret < 0, 0, _("Failed to build message\n"));
 	va_end(ap);
 
@@ -213,6 +213,7 @@ unsigned io_fprintln(const char *fname, const char *fmt, ...)
 	RETVAL_IF(ret != 0, 0, _("Failed to close \"%s\" - %s\n"),
 		  fname, strerror(errno));
 
+	mem_free(buf);
 	return 1;
 }
 
@@ -225,10 +226,7 @@ unsigned io_fprintln(const char *fname, const char *fmt, ...)
  */
 void io_init(const char *cfile, const char *datadir)
 {
-	FILE *data_file;
 	const char *home;
-	char apts_file[BUFSIZ] = "";
-	int ch;
 
 	if (datadir != NULL) {
 		home = datadir;
@@ -264,41 +262,9 @@ void io_init(const char *cfile, const char *datadir)
 			snprintf(path_apts, BUFSIZ, "%s/" APTS_PATH, home);
 		}
 	} else {
-		snprintf(apts_file, BUFSIZ, "%s", cfile);
-		strncpy(path_apts, apts_file, BUFSIZ);
-		/* check if the file exists, otherwise create it */
-		data_file = fopen(path_apts, "r");
-		if (data_file == NULL) {
-			printf(_("%s does not exist, create it now [y/n]? "),
-			       path_apts);
-			ch = getchar();
-			switch (ch) {
-			case 'N':
-			case 'n':
-				puts(_("aborting...\n"));
-				exit_calcurse(EXIT_FAILURE);
-				break;
-
-			case 'Y':
-			case 'y':
-				data_file = fopen(path_apts, "w");
-				if (data_file == NULL) {
-					perror(path_apts);
-					exit_calcurse(EXIT_FAILURE);
-				} else {
-					printf(_("%s successfully created\n"),
-					       path_apts);
-					puts(_("starting interactive mode...\n"));
-				}
-				break;
-
-			default:
-				puts(_("aborting...\n"));
-				exit_calcurse(EXIT_FAILURE);
-				break;
-			}
-		}
-		file_close(data_file, __FILE_POS__);
+		snprintf(path_apts, BUFSIZ, "%s", cfile);
+		EXIT_IF(!io_file_exists(path_apts), _("%s does not exist"),
+			path_apts);
 	}
 }
 
@@ -308,7 +274,7 @@ void io_extract_data(char *dst_data, const char *org, int len)
 
 	for (; *org == ' ' || *org == '\t'; org++) ;
 	for (i = 0; i < len - 1; i++) {
-		if (*org == '\n' || *org == '\0' || *org == '#')
+		if (*org == '\n' || *org == '\0')
 			break;
 		*dst_data++ = *org++;
 	}
@@ -317,21 +283,35 @@ void io_extract_data(char *dst_data, const char *org, int len)
 
 static pthread_mutex_t io_save_mutex = PTHREAD_MUTEX_INITIALIZER;
 
+void io_save_mutex_lock(void)
+{
+	pthread_mutex_lock(&io_save_mutex);
+}
+
+void io_save_mutex_unlock(void)
+{
+	pthread_mutex_unlock(&io_save_mutex);
+}
+
 /*
  * Save the apts data file, which contains the
  * appointments first, and then the events.
  * Recursive items are written first.
  */
-unsigned io_save_apts(void)
+unsigned io_save_apts(const char *aptsfile)
 {
 	llist_item_t *i;
 	FILE *fp;
 
-	if (read_only)
-		return 1;
+	if (aptsfile) {
+		if (read_only)
+			return 1;
 
-	if ((fp = fopen(path_apts, "w")) == NULL)
-		return 0;
+		if ((fp = fopen(aptsfile, "w")) == NULL)
+			return 0;
+	} else {
+		fp = stdout;
+	}
 
 	recur_save_data(fp);
 
@@ -348,28 +328,36 @@ unsigned io_save_apts(void)
 		struct event *ev = LLIST_TS_GET_DATA(i);
 		event_write(ev, fp);
 	}
-	file_close(fp, __FILE_POS__);
+
+	if (aptsfile)
+		file_close(fp, __FILE_POS__);
 
 	return 1;
 }
 
 /* Save the todo data file. */
-unsigned io_save_todo(void)
+unsigned io_save_todo(const char *todofile)
 {
 	llist_item_t *i;
 	FILE *fp;
 
-	if (read_only)
-		return 1;
+	if (todofile) {
+		if (read_only)
+			return 1;
 
-	if ((fp = fopen(path_todo, "w")) == NULL)
-		return 0;
+		if ((fp = fopen(todofile, "w")) == NULL)
+			return 0;
+	} else {
+		fp = stdout;
+	}
 
 	LLIST_FOREACH(&todolist, i) {
 		struct todo *todo = LLIST_TS_GET_DATA(i);
 		todo_write(todo, fp);
 	}
-	file_close(fp, __FILE_POS__);
+
+	if (todofile)
+		file_close(fp, __FILE_POS__);
 
 	return 1;
 }
@@ -417,12 +405,12 @@ void io_save_cal(enum save_display display)
 
 	if (show_bar)
 		progress_bar(PROGRESS_BAR_SAVE, PROGRESS_BAR_TODO);
-	if (!io_save_todo())
+	if (!io_save_todo(path_todo))
 		ERROR_MSG("%s", access_pb);
 
 	if (show_bar)
 		progress_bar(PROGRESS_BAR_SAVE, PROGRESS_BAR_APTS);
-	if (!io_save_apts())
+	if (!io_save_apts(path_apts))
 		ERROR_MSG("%s", access_pb);
 
 	if (show_bar)
@@ -430,8 +418,11 @@ void io_save_cal(enum save_display display)
 	if (!io_save_keys())
 		ERROR_MSG("%s", access_pb);
 
+	io_unset_modified();
+
 	/* Print a message telling data were saved */
-	if (ui_mode == UI_CURSES && conf.system_dialogs) {
+	if (ui_mode == UI_CURSES && display == IO_SAVE_DISPLAY_BAR &&
+	    conf.system_dialogs) {
 		status_mesg(save_success, enter);
 		wgetch(win[KEY].p);
 	}
@@ -450,7 +441,7 @@ static void io_load_error(const char *filename, unsigned line,
  * and then load either: a new appointment, a new event, or a new
  * recursive item (which can also be either an event or an appointment).
  */
-void io_load_app(void)
+void io_load_app(struct item_filter *filter)
 {
 	FILE *data_file;
 	int c, is_appointment, is_event, is_recursive;
@@ -606,18 +597,19 @@ void io_load_app(void)
 			if (is_recursive) {
 				recur_apoint_scan(data_file, start, end,
 						  type, freq, until, notep,
-						  &exc, state);
+						  &exc, state, filter);
 			} else {
 				apoint_scan(data_file, start, end, state,
-					    notep);
+					    notep, filter);
 			}
 		} else if (is_event) {
 			if (is_recursive) {
 				recur_event_scan(data_file, start, id,
 						 type, freq, until, notep,
-						 &exc);
+						 &exc, filter);
 			} else {
-				event_scan(data_file, start, id, notep);
+				event_scan(data_file, start, id, notep,
+					   filter);
 			}
 		} else {
 			io_load_error(path_apts, line,
@@ -629,7 +621,7 @@ void io_load_app(void)
 }
 
 /* Load the todo data */
-void io_load_todo(void)
+void io_load_todo(struct item_filter *filter)
 {
 	FILE *data_file;
 	char *newline;
@@ -672,12 +664,130 @@ void io_load_todo(void)
 		if (newline)
 			*newline = '\0';
 		io_extract_data(e_todo, buf, sizeof buf);
+
+		/* Filter item. */
+		if (filter) {
+			if (!(filter->type_mask & TYPE_MASK_TODO))
+				continue;
+			if (filter->regex &&
+			    regexec(filter->regex, e_todo, 0, 0, 0))
+				continue;
+			if (filter->priority && id != filter->priority)
+				continue;
+			if (filter->completed && id > 0)
+				continue;
+			if (filter->uncompleted && id < 0)
+				continue;
+		}
+
 		todo_add(e_todo, id, note);
 		++nb_tod;
 	}
 	file_close(data_file, __FILE_POS__);
 }
 
+/* Load appointments and todo items */
+void io_load_data(struct item_filter *filter)
+{
+	io_load_app(filter);
+	io_load_todo(filter);
+}
+
+void io_reload_data(void)
+{
+	char *msg_um_asktype = NULL;
+	const char *reload_success =
+		_("The data files were reloaded successfully");
+	const char *enter = _("Press [ENTER] to continue");
+
+	if (io_get_modified()) {
+		const char *msg_um_prefix =
+				_("There are unsaved modifications:");
+		const char *msg_um_discard = _("(d)iscard");
+		const char *msg_um_merge = _("(m)erge");
+		const char *msg_um_keep = _("(k)eep and cancel");
+		const char *msg_um_choice = _("[dmk]");
+
+		asprintf(&msg_um_asktype, "%s %s, %s, %s", msg_um_prefix,
+			 msg_um_discard, msg_um_merge, msg_um_keep);
+
+		char *path_apts_backup, *path_todo_backup;
+		const char *backup_ext = ".sav";
+
+		switch (status_ask_choice(msg_um_asktype, msg_um_choice, 3)) {
+		case 1:
+			break;
+		case 2:
+			asprintf(&path_apts_backup, "%s%s", path_apts,
+				 backup_ext);
+			asprintf(&path_todo_backup, "%s%s", path_todo,
+				 backup_ext);
+
+			io_save_mutex_lock();
+			io_save_apts(path_apts_backup);
+			io_save_todo(path_todo_backup);
+			io_save_mutex_unlock();
+
+			if (!io_files_equal(path_apts, path_apts_backup)) {
+				const char *arg_apts[] = { conf.mergetool,
+							   path_apts,
+							   path_apts_backup,
+							   NULL };
+				wins_launch_external(arg_apts);
+			}
+
+			if (!io_files_equal(path_todo, path_todo_backup)) {
+				const char *arg_todo[] = { conf.mergetool,
+							   path_todo,
+							   path_todo_backup,
+							   NULL };
+				wins_launch_external(arg_todo);
+			}
+
+			xfree(path_apts_backup);
+			xfree(path_todo_backup);
+			break;
+		case 3:
+			/* FALLTHROUGH */
+		default:
+			wins_update(FLAG_STA);
+			goto cleanup;
+		}
+	}
+
+	if (notify_bar())
+		notify_stop_main_thread();
+
+	/* Reinitialize data structures. */
+	apoint_llist_free();
+	event_llist_free();
+	recur_apoint_llist_free();
+	recur_event_llist_free();
+	todo_free_list();
+
+	apoint_llist_init();
+	event_llist_init();
+	recur_apoint_llist_init();
+	recur_event_llist_init();
+	todo_init_list();
+
+	io_load_data(NULL);
+	io_unset_modified();
+	ui_todo_load_items();
+	ui_todo_sel_reset();
+
+	if (conf.system_dialogs) {
+		status_mesg(reload_success, enter);
+		wgetch(win[KEY].p);
+	}
+
+	if (notify_bar())
+		notify_start_main_thread();
+
+cleanup:
+	mem_free(msg_um_asktype);
+}
+
 static void
 load_keys_ht_getkey(struct ht_keybindings_s *data, const char **key,
 		    int *len)
@@ -794,14 +904,14 @@ void io_load_keys(const char *pager)
 				int ch;
 
 				if ((ch = keys_str2int(key_ch)) < 0) {
-					char unknown_key[BUFSIZ];
+					char *unknown_key;
 
 					skipped++;
-					(void)snprintf(unknown_key, BUFSIZ,
-						       _("Error reading key: \"%s\""),
-						       key_ch);
-					io_log_print(log, line,
-						     unknown_key);
+					asprintf(&unknown_key,
+						 _("Error reading key: \"%s\""),
+						 key_ch);
+					io_log_print(log, line, unknown_key);
+					mem_free(unknown_key);
 				} else {
 					int used;
 
@@ -810,18 +920,15 @@ void io_load_keys(const char *pager)
 								ht_elm->
 								key);
 					if (used) {
-						char already_assigned
-						    [BUFSIZ];
+						char *already_assigned;
 
 						skipped++;
-						(void)
-						    snprintf
-						    (already_assigned,
-						     BUFSIZ,
-						     _("\"%s\" assigned multiple times!"),
-						     key_ch);
+						asprintf(&already_assigned,
+							 _("\"%s\" assigned multiple times!"),
+							 key_ch);
 						io_log_print(log, line,
 							     already_assigned);
+						mem_free(already_assigned);
 					} else {
 						assigned++;
 					}
@@ -1032,8 +1139,8 @@ static FILE *get_import_stream(enum import_type type)
 void io_import_data(enum import_type type, const char *stream_name)
 {
 	const char *proc_report =
-	    _("Import process report: %04d lines read ");
-	char stats_str[4][BUFSIZ];
+	    _("Import process report: %04d lines read");
+	char *stats_str[4];
 	FILE *stream = NULL;
 	struct io_file *log;
 	struct {
@@ -1077,25 +1184,25 @@ void io_import_data(enum import_type type, const char *stream_name)
 	if (stream != stdin)
 		file_close(stream, __FILE_POS__);
 
-	snprintf(stats_str[0], BUFSIZ,
-		 ngettext("%d app", "%d apps", stats.apoints),
+	asprintf(&stats_str[0], ngettext("%d app", "%d apps", stats.apoints),
 		 stats.apoints);
-	snprintf(stats_str[1], BUFSIZ,
+	asprintf(&stats_str[1],
 		 ngettext("%d event", "%d events", stats.events),
 		 stats.events);
-	snprintf(stats_str[2], BUFSIZ,
-		 ngettext("%d todo", "%d todos", stats.todos),
+	asprintf(&stats_str[2], ngettext("%d todo", "%d todos", stats.todos),
 		 stats.todos);
-	snprintf(stats_str[3], BUFSIZ, _("%d skipped"), stats.skipped);
+	asprintf(&stats_str[3], _("%d skipped"), stats.skipped);
 
 	if (ui_mode == UI_CURSES && conf.system_dialogs) {
-		char read[BUFSIZ], stat[BUFSIZ];
+		char *read, *stat;
 
-		snprintf(read, BUFSIZ, proc_report, stats.lines);
-		snprintf(stat, BUFSIZ, "%s / %s / %s / %s (%s)",
+		asprintf(&read, proc_report, stats.lines);
+		asprintf(&stat, "%s / %s / %s / %s (%s)",
 			 stats_str[0], stats_str[1], stats_str[2],
 			 stats_str[3], _("Press [ENTER] to continue"));
 		status_mesg(read, stat);
+		mem_free(read);
+		mem_free(stat);
 		wgetch(win[KEY].p);
 	} else if (ui_mode == UI_CMDLINE) {
 		printf(proc_report, stats.lines);
@@ -1113,31 +1220,43 @@ void io_import_data(enum import_type type, const char *stream_name)
 
 		io_log_display(log, view_log, conf.pager);
 	}
+
+	mem_free(stats_str[0]);
+	mem_free(stats_str[1]);
+	mem_free(stats_str[2]);
+	mem_free(stats_str[3]);
 	io_log_free(log);
 }
 
 struct io_file *io_log_init(void)
 {
-	char logprefix[BUFSIZ];
-	char *logname;
-	struct io_file *log;
+	char *logprefix, *logname;
+	struct io_file *log = mem_malloc(sizeof(struct io_file));
 
-	snprintf(logprefix, BUFSIZ, "%s/calcurse_log.", get_tempdir());
-	logname = new_tempfile(logprefix, TMPEXTSIZ);
-	RETVAL_IF(logname == NULL, 0,
-		  _("Warning: could not create temporary log file, Aborting..."));
-	log = mem_malloc(sizeof(struct io_file));
-	RETVAL_IF(log == NULL, 0,
-		  _("Warning: could not open temporary log file, Aborting..."));
-	snprintf(log->name, sizeof(log->name), "%s%s", logprefix, logname);
-	mem_free(logname);
+	if (!log) {
+		ERROR_MSG(_("Warning: could not open temporary log file, Aborting..."));
+		return NULL;
+	}
+	asprintf(&logprefix, "%s/calcurse_log", get_tempdir());
+	logname = new_tempfile(logprefix);
+	if (!logname) {
+		ERROR_MSG(_("Warning: could not create temporary log file, Aborting..."));
+		goto error;
+	}
+	strncpy(log->name, logname, sizeof(log->name));
 	log->fd = fopen(log->name, "w");
 	if (log->fd == NULL) {
 		ERROR_MSG(_("Warning: could not open temporary log file, Aborting..."));
-		mem_free(log);
-		return 0;
+		goto error;
 	}
 
+	goto cleanup;
+error:
+	mem_free(log);
+	log = NULL;
+cleanup:
+	mem_free(logprefix);
+	mem_free(logname);
 	return log;
 }
 
@@ -1153,16 +1272,19 @@ void io_log_display(struct io_file *log, const char *msg,
 	RETURN_IF(log == NULL, _("No log file to display!"));
 	if (ui_mode == UI_CMDLINE) {
 		printf("\n%s [y/n] ", msg);
-		if (fgetc(stdin) == 'y') {
-			const char *arg[] = { pager, log->name, NULL };
-			int pid;
+		if (fgetc(stdin) != 'y')
+			return;
 
-			if ((pid = fork_exec(NULL, NULL, pager, arg)))
-				child_wait(NULL, NULL, pid);
-		}
+		const char *arg[] = { pager, log->name, NULL };
+		int pid;
+
+		if ((pid = fork_exec(NULL, NULL, pager, arg)))
+			child_wait(NULL, NULL, pid);
 	} else {
-		if (status_ask_bool(msg) == 1)
-			wins_launch_external(log->name, pager);
+		if (status_ask_bool(msg) == 1) {
+			const char *arg[] = { pager, log->name, NULL };
+			wins_launch_external(arg);
+		}
 		wins_erase_status_bar();
 	}
 }
@@ -1200,10 +1322,14 @@ void io_start_psave_thread(void)
 /* Stop periodic data saves. */
 void io_stop_psave_thread(void)
 {
-	if (io_t_psave) {
-		pthread_cancel(io_t_psave);
-		pthread_join(io_t_psave, NULL);
-	}
+	if (!io_t_psave)
+		return;
+
+	/* Lock the mutex to avoid cancelling the thread during saving. */
+	io_save_mutex_lock();
+	pthread_cancel(io_t_psave);
+	pthread_join(io_t_psave, NULL);
+	io_save_mutex_unlock();
 }
 
 /*
@@ -1241,11 +1367,11 @@ void io_set_lock(void)
 			 "lock file: \n\"%s\"\n"
 			 "and restart calcurse.\n"), path_cpid);
 		exit(EXIT_FAILURE);
-	} else {
-		if (!io_dump_pid(path_cpid))
-			EXIT(_("FATAL ERROR: could not create %s: %s\n"),
-			     path_cpid, strerror(errno));
 	}
+
+	if (!io_dump_pid(path_cpid))
+		EXIT(_("FATAL ERROR: could not create %s: %s\n"),
+		     path_cpid, strerror(errno));
 }
 
 /*
@@ -1298,18 +1424,40 @@ unsigned io_get_pid(char *file)
 int io_file_is_empty(char *file)
 {
 	FILE *fp;
+	int ret = -1;
 
 	if (file && (fp = fopen(file, "r"))) {
-		if ((fgetc(fp) == '\n' && fgetc(fp) == EOF) || feof(fp)) {
-			fclose(fp);
-			return 1;
-		} else {
-			fclose(fp);
-			return 0;
-		}
+		ret = (fgetc(fp) == '\n' && fgetc(fp) == EOF) || feof(fp);
+		fclose(fp);
 	}
 
-	return -1;
+	return ret;
+}
+
+/*
+ * Check whether two files are equal.
+ */
+int io_files_equal(const char *file1, const char *file2)
+{
+	FILE *fp1, *fp2;
+	int ret = 0;
+
+	if (!file1 || !file2)
+		return 0;
+
+	fp1 = fopen(file1, "rb");
+	fp2 = fopen(file2, "rb");
+
+	while (!feof(fp1) && !feof(fp2)) {
+		if (fgetc(fp1) != fgetc(fp2))
+			goto cleanup;
+	}
+
+	ret = 1;
+cleanup:
+	fclose(fp1);
+	fclose(fp2);
+	return ret;
 }
 
 /*
@@ -1342,3 +1490,18 @@ int io_file_cp(const char *src, const char *dst)
 
 	return 1;
 }
+
+void io_unset_modified(void)
+{
+	modified = 0;
+}
+
+void io_set_modified(void)
+{
+	modified = 1;
+}
+
+int io_get_modified(void)
+{
+	return modified;
+}
