@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,59 +44,62 @@
 struct keydef_s {
 	const char *label;
 	const char *binding;
+	const char *sb_label;
 };
 
 static llist_t keys[NBKEYS];
 static enum key actions[MAXKEYVAL];
 
+#define gettext_noop(s) s
 static struct keydef_s keydef[NBKEYS] = {
-	{"generic-cancel", "ESC"},
-	{"generic-select", "SPC"},
-	{"generic-credits", "@"},
-	{"generic-help", "?"},
-	{"generic-quit", "q Q"},
-	{"generic-save", "s S C-s"},
-	{"generic-copy", "c"},
-	{"generic-paste", "p C-v"},
-	{"generic-change-view", "TAB"},
-	{"generic-import", "i I"},
-	{"generic-export", "x X"},
-	{"generic-goto", "g G"},
-	{"generic-other-cmd", "o O"},
-	{"generic-config-menu", "C"},
-	{"generic-redraw", "C-r"},
-	{"generic-add-appt", "C-a"},
-	{"generic-add-todo", "C-t"},
-	{"generic-prev-day", "T C-h"},
-	{"generic-next-day", "t C-l"},
-	{"generic-prev-week", "W C-k"},
-	{"generic-next-week", "w C-j"},
-	{"generic-prev-month", "M"},
-	{"generic-next-month", "m"},
-	{"generic-prev-year", "Y"},
-	{"generic-next-year", "y"},
-	{"generic-scroll-down", "C-n"},
-	{"generic-scroll-up", "C-p"},
-	{"generic-goto-today", "C-g"},
-	{"generic-command", ":"},
+	{ "generic-cancel", "ESC", gettext_noop("Cancel") },
+	{ "generic-select", "SPC", gettext_noop("Select") },
+	{ "generic-credits", "@", gettext_noop("Credits") },
+	{ "generic-help", "?", gettext_noop("Help") },
+	{ "generic-quit", "q Q", gettext_noop("Quit") },
+	{ "generic-save", "s S C-s", gettext_noop("Save") },
+	{ "generic-reload", "R", gettext_noop("Reload") },
+	{ "generic-copy", "c", gettext_noop("Copy") },
+	{ "generic-paste", "p C-v", gettext_noop("Paste") },
+	{ "generic-change-view", "TAB", gettext_noop("Chg Win") },
+	{ "generic-import", "i I", gettext_noop("Import") },
+	{ "generic-export", "x X", gettext_noop("Export") },
+	{ "generic-goto", "g G", gettext_noop("Go to") },
+	{ "generic-other-cmd", "o O", gettext_noop("OtherCmd") },
+	{ "generic-config-menu", "C", gettext_noop("Config") },
+	{ "generic-redraw", "C-r", gettext_noop("Redraw") },
+	{ "generic-add-appt", "C-a", gettext_noop("Add Appt") },
+	{ "generic-add-todo", "C-t", gettext_noop("Add Todo") },
+	{ "generic-prev-day", "T C-h", gettext_noop("-1 Day") },
+	{ "generic-next-day", "t C-l", gettext_noop("+1 Day") },
+	{ "generic-prev-week", "W C-k", gettext_noop("-1 Week") },
+	{ "generic-next-week", "w C-j", gettext_noop("+1 Week") },
+	{ "generic-prev-month", "M", gettext_noop("-1 Month") },
+	{ "generic-next-month", "m", gettext_noop("+1 Month") },
+	{ "generic-prev-year", "Y", gettext_noop("-1 Year") },
+	{ "generic-next-year", "y", gettext_noop("+1 Year") },
+	{ "generic-scroll-down", "C-n", gettext_noop("Nxt View") },
+	{ "generic-scroll-up", "C-p", gettext_noop("Prv View") },
+	{ "generic-goto-today", "C-g", gettext_noop("Today") },
+	{ "generic-command", ":", gettext_noop("Command") },
 
-	{"move-right", "l L RGT"},
-	{"move-left", "h H LFT"},
-	{"move-down", "j J DWN"},
-	{"move-up", "k K UP"},
-	{"start-of-week", "0"},
-	{"end-of-week", "$"},
-	{"add-item", "a A"},
-	{"del-item", "d D"},
-	{"edit-item", "e E"},
-	{"view-item", "v V"},
-	{"pipe-item", "|"},
-	{"flag-item", "!"},
-	{"repeat", "r R"},
-	{"edit-note", "n N"},
-	{"view-note", ">"},
-	{"raise-priority", "+"},
-	{"lower-priority", "-"},
+	{ "move-right", "l L RGT", gettext_noop("Right") },
+	{ "move-left", "h H LFT", gettext_noop("Left") },
+	{ "move-down", "j J DWN", gettext_noop("Down") },
+	{ "move-up", "k K UP", gettext_noop("Up") },
+	{ "start-of-week", "0", gettext_noop("beg Week") },
+	{ "end-of-week", "$", gettext_noop("end Week") },
+	{ "add-item", "a A", gettext_noop("Add Item") },
+	{ "del-item", "d D", gettext_noop("Del Item") },
+	{ "edit-item", "e E", gettext_noop("Edit Itm") },
+	{ "view-item", "v V", gettext_noop("View") },
+	{ "pipe-item", "|", gettext_noop("Pipe") },
+	{ "flag-item", "!", gettext_noop("Flag Itm") },
+	{ "repeat", "r", gettext_noop("Repeat") },
+	{ "edit-note", "n N", gettext_noop("EditNote") },
+	{ "view-note", ">", gettext_noop("ViewNote") },
+	{ "raise-priority", "+", gettext_noop("Prio.+") },
+	{ "lower-priority", "-", gettext_noop("Prio.-") }
 };
 
 static void dump_intro(FILE * fd)
@@ -410,10 +413,11 @@ static char *keys_format_label(char *key, int keylen)
 }
 
 void
-keys_display_bindings_bar(WINDOW * win, struct binding *bindings[],
-			  int count, int page_base, int page_size,
-			  struct binding *more)
+keys_display_bindings_bar(WINDOW * win, int *bindings, int count,
+			  int page_base, int page_size)
 {
+	page_size = MIN(page_size, count - page_base);
+
 	/* Padding between two key bindings. */
 	const int padding =
 	    (col * 2) / page_size - (KEYS_KEYLEN + KEYS_LABELEN + 1);
@@ -430,24 +434,60 @@ keys_display_bindings_bar(WINDOW * win, struct binding *bindings[],
 		const int label_pos_x = key_pos_x + KEYS_KEYLEN + 1;
 		const int label_pos_y = key_pos_y;
 
-		struct binding *binding;
 		char key[KEYS_KEYLEN + 1], *fmtkey;
 
-		if (!more || i < page_size - 1
-		    || page_base + i == count - 1)
-			binding = bindings[page_base + i];
-		else
-			binding = more;
+		int binding_key;
 
-		strncpy(key, keys_action_firstkey(binding->action),
-			KEYS_KEYLEN);
-		key[KEYS_KEYLEN] = '\0';
-		fmtkey = keys_format_label(key, KEYS_KEYLEN);
+		if (i < page_size - 1 || page_base + i == count - 1)
+			binding_key = bindings[page_base + i];
+		else
+			binding_key = KEY_GENERIC_OTHER_CMD;
+
+		const char *label;
+
+		if (binding_key < NBKEYS) {
+			strncpy(key, keys_action_firstkey(binding_key),
+				KEYS_KEYLEN);
+			key[KEYS_KEYLEN] = '\0';
+			label = gettext(keydef[binding_key].sb_label);
+		} else {
+			switch (binding_key) {
+			case KEY_CONFIGMENU_GENERAL:
+				strcpy(key, "g");
+				label = _("General");
+				break;
+			case KEY_CONFIGMENU_LAYOUT:
+				strcpy(key, "l");
+				label = _("Layout");
+				break;
+			case KEY_CONFIGMENU_SIDEBAR:
+				strcpy(key, "s");
+				label = _("Sidebar");
+				break;
+			case KEY_CONFIGMENU_COLOR:
+				strcpy(key, "c");
+				label = _("Color");
+				break;
+			case KEY_CONFIGMENU_NOTIFY:
+				strcpy(key, "n");
+				label = _("Notify");
+				break;
+			case KEY_CONFIGMENU_KEYS:
+				strcpy(key, "k");
+				label = _("Keys");
+				break;
+			default:
+				strcpy(key, "?");
+				label = _("Unknown");
+				break;
+			}
+		}
 
 		custom_apply_attr(win, ATTR_HIGHEST);
+		fmtkey = keys_format_label(key, KEYS_KEYLEN);
 		mvwaddstr(win, key_pos_y, key_pos_x, fmtkey);
 		custom_remove_attr(win, ATTR_HIGHEST);
-		mvwaddstr(win, label_pos_y, label_pos_x, binding->label);
+		mvwaddstr(win, label_pos_y, label_pos_x, label);
 	}
 	wnoutrefresh(win);
 }
@@ -470,6 +510,7 @@ void keys_popup_info(enum key key)
 	info[KEY_GENERIC_QUIT] =
 	    _("Exit from the current menu, or quit calcurse.");
 	info[KEY_GENERIC_SAVE] = _("Save calcurse data.");
+	info[KEY_GENERIC_RELOAD] = _("Reload appointments and todo items.");
 	info[KEY_GENERIC_COPY] =
 	    _("Copy the item that is currently selected.");
 	info[KEY_GENERIC_PASTE] =
