@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -321,6 +321,15 @@ void notify_update_bar(void)
 	pthread_mutex_unlock(&notify.mutex);
 }
 
+static void
+notify_main_thread_cleanup(void *arg)
+{
+	pthread_mutex_trylock(&notify.mutex);
+	pthread_mutex_unlock(&notify.mutex);
+	pthread_mutex_trylock(&nbar.mutex);
+	pthread_mutex_unlock(&nbar.mutex);
+}
+
 /* Update the notication bar content */
 /* ARGSUSED0 */
 static void *notify_main_thread(void *arg)
@@ -334,6 +343,8 @@ static void *notify_main_thread(void *arg)
 
 	elapse = 0;
 
+	pthread_cleanup_push(notify_main_thread_cleanup, NULL);
+
 	for (;;) {
 		ntimer = time(NULL);
 		localtime_r(&ntimer, &ntime);
@@ -357,6 +368,8 @@ static void *notify_main_thread(void *arg)
 				notify_check_next_app(0);
 		}
 	}
+
+	pthread_cleanup_pop(0);
 	pthread_exit(NULL);
 }
 
@@ -680,6 +693,7 @@ static void config_option_edit(int i)
 	const char *cmd_str = _("Enter the notification command ");
 
 	buf = mem_malloc(BUFSIZ);
+	buf[0] = '\0';
 
 	switch (i) {
 	case 0:
@@ -762,6 +776,9 @@ static void config_option_edit(int i)
 /* Notify-bar configuration. */
 void notify_config_bar(void)
 {
+	static int bindings[] = {
+		KEY_GENERIC_QUIT, KEY_MOVE_UP, KEY_MOVE_DOWN, KEY_EDIT_ITEM
+	};
 	struct listbox lb;
 	int ch;
 
@@ -771,8 +788,12 @@ void notify_config_bar(void)
 		     config_option_height, print_config_option);
 	listbox_load_items(&lb, 8);
 	listbox_draw_deco(&lb, 0);
-	status_mesg("", "");
 	listbox_display(&lb);
+	wins_set_bindings(bindings, ARRAY_SIZE(bindings));
+	wins_status_bar();
+	wnoutrefresh(win[STA].p);
+	wmove(win[STA].p, 0, 0);
+	wins_doupdate();
 
 	while ((ch = keys_getch(win[KEY].p, NULL, NULL)) != KEY_GENERIC_QUIT) {
 		switch (ch) {
@@ -805,8 +826,11 @@ void notify_config_bar(void)
 			clearok(curscr, TRUE);
 		}
 
-		status_mesg("", "");
 		listbox_display(&lb);
+		wins_status_bar();
+		wnoutrefresh(win[STA].p);
+		wmove(win[STA].p, 0, 0);
+		wins_doupdate();
 	}
 
 	listbox_delete(&lb);
