@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -86,6 +86,10 @@ static void generic_hdlr(int sig)
 		}
 		exit(EXIT_SUCCESS);
 		break;
+	case SIGUSR1:
+		want_reload = 1;
+		ungetch(KEY_RESIZE);
+		break;
 	}
 }
 
@@ -112,6 +116,7 @@ void sigs_init()
 	if (!sigs_set_hdlr(SIGCHLD, generic_hdlr)
 	    || !sigs_set_hdlr(SIGWINCH, generic_hdlr)
 	    || !sigs_set_hdlr(SIGTERM, generic_hdlr)
+	    || !sigs_set_hdlr(SIGUSR1, generic_hdlr)
 	    || !sigs_set_hdlr(SIGINT, SIG_IGN))
 		exit_calcurse(1);
 }
