@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -309,6 +309,10 @@ draw_monthly_view(struct scrollwin *sw, struct date *current_day,
 
 	/* Write the current month and year on top of the calendar */
 	WINS_CALENDAR_LOCK;
+	if (yr * YEARINMONTHS + mo != monthly_view_cache_month) {
+		/* erase the window if a new month is selected */
+		werase(sw_cal.inner);
+	}
 	custom_apply_attr(sw->inner, ATTR_HIGHEST);
 	mvwprintw(sw->inner, ofs_y,
 		  (w - (strlen(_(monthnames[mo - 1])) + 5)) / 2,
@@ -491,6 +495,7 @@ draw_weekly_view(struct scrollwin *sw, struct date *current_day,
 	/* Print the week number. */
 	weeknum = ISO8601weeknum(&t);
 	WINS_CALENDAR_LOCK;
+	werase(sw_cal.inner);
 	custom_apply_attr(sw->inner, ATTR_HIGHEST);
 	mvwprintw(sw->win, conf.compact_panels ? 0 : 2, sw->w - 9,
 		  "(# %02d)", weeknum);
@@ -596,15 +601,8 @@ void ui_calendar_update_panel(void)
 	unsigned sunday_first;
 
 	ui_calendar_store_current_date(&current_day);
-
-	WINS_CALENDAR_LOCK;
-	werase(sw_cal.inner);
-	WINS_CALENDAR_UNLOCK;
-
-	sunday_first = ui_calendar_week_begins_on_monday()? 0 : 1;
-
+	sunday_first = !ui_calendar_week_begins_on_monday();
 	draw_calendar[ui_calendar_view] (&sw_cal, &current_day, sunday_first);
-
 	wins_scrollwin_display(&sw_cal);
 }
 
@@ -629,20 +627,18 @@ void ui_calendar_change_day(int datefmt)
 {
 #define LDAY 11
 	char selected_day[LDAY] = "";
-	char outstr[BUFSIZ];
+	char *outstr;
 	int dday, dmonth, dyear;
 	int wrong_day = 1;
-	const char *mesg_line1 =
-	    _("The day you entered is not valid "
-	      "(should be between 01/01/1902 and 12/31/2037)");
+	const char *mesg_line1 = _("The day you entered is not valid");
 	const char *mesg_line2 = _("Press [ENTER] to continue");
 	const char *request_date =
 	    _("Enter the day to go to [ENTER for today] : %s");
 
 	while (wrong_day) {
-		snprintf(outstr, BUFSIZ, request_date,
-			 DATEFMT_DESC(datefmt));
+		asprintf(&outstr, request_date, DATEFMT_DESC(datefmt));
 		status_mesg(outstr, "");
+		mem_free(outstr);
 		if (getstring(win[STA].p, selected_day, LDAY, 0, 1) ==
 		    GETSTRING_ESC) {
 			return;
@@ -741,10 +737,6 @@ void ui_calendar_move(enum move move, int count)
 			t.tm_mday = 1;
 			t.tm_mon = 0;
 			t.tm_year = 2;
-		} else if (t.tm_year > 137) {
-			t.tm_mday = 31;
-			t.tm_mon = 11;
-			t.tm_year = 137;
 		}
 
 		slctd_day.dd = t.tm_mday;
