@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -156,8 +156,7 @@ static void update_rept(struct rpt **rpt, const long start)
 {
 	int newtype, newfreq;
 	long newuntil;
-	char outstr[BUFSIZ];
-	char *freqstr, *timstr;
+	char *freqstr = NULL, *timstr, *outstr;
 	const char *msg_rpt_prefix = _("Enter the new repetition type:");
 	const char *msg_rpt_daily = _("(d)aily");
 	const char *msg_rpt_weekly = _("(w)eekly");
@@ -166,7 +165,7 @@ static void update_rept(struct rpt **rpt, const long start)
 
 	/* Find the current repetition type. */
 	const char *rpt_current;
-	char msg_rpt_current[BUFSIZ];
+	char *msg_rpt_current;
 	switch (recur_def2char((*rpt)->type)) {
 	case 'D':
 		rpt_current = msg_rpt_daily;
@@ -185,15 +184,12 @@ static void update_rept(struct rpt **rpt, const long start)
 		rpt_current = msg_rpt_daily;
 	}
 
-	snprintf(msg_rpt_current, BUFSIZ, _("(currently using %s)"),
-		 rpt_current);
+	asprintf(&msg_rpt_current, _("(currently using %s)"), rpt_current);
 
-	char msg_rpt_asktype[BUFSIZ];
-	snprintf(msg_rpt_asktype, BUFSIZ, "%s %s, %s, %s, %s ? %s",
-		 msg_rpt_prefix,
-		 msg_rpt_daily,
-		 msg_rpt_weekly, msg_rpt_monthly, msg_rpt_yearly,
-		 msg_rpt_current);
+	char *msg_rpt_asktype;
+	asprintf(&msg_rpt_asktype, "%s %s, %s, %s, %s ? %s", msg_rpt_prefix,
+		 msg_rpt_daily, msg_rpt_weekly, msg_rpt_monthly,
+		 msg_rpt_yearly, msg_rpt_current);
 
 	const char *msg_rpt_choice = _("[dwmy]");
 	const char *msg_wrong_freq =
@@ -219,20 +215,18 @@ static void update_rept(struct rpt **rpt, const long start)
 		newtype = 'Y';
 		break;
 	default:
-		return;
+		goto cleanup;
 	}
 
 	do {
 		status_mesg(_("Enter the new repetition frequence:"), "");
-		freqstr = mem_malloc(BUFSIZ);
-		snprintf(freqstr, BUFSIZ, "%d", (*rpt)->freq);
+		mem_free(freqstr);
+		asprintf(&freqstr, "%d", (*rpt)->freq);
 		if (updatestring(win[STA].p, &freqstr, 0, 1) !=
 		    GETSTRING_VALID) {
-			mem_free(freqstr);
-			return;
+			goto cleanup;
 		}
 		newfreq = atoi(freqstr);
-		mem_free(freqstr);
 		if (newfreq == 0) {
 			status_mesg(msg_wrong_freq, msg_enter);
 			wgetch(win[KEY].p);
@@ -246,17 +240,16 @@ static void update_rept(struct rpt **rpt, const long start)
 		struct date new_date;
 		int newmonth, newday, newyear;
 
-		snprintf(outstr, BUFSIZ,
-			 _("Enter the new ending date: [%s] or '0'"),
+		asprintf(&outstr, _("Enter the new ending date: [%s] or '0'"),
 			 DATEFMT_DESC(conf.input_datefmt));
 		status_mesg(outstr, "");
+		mem_free(outstr);
 		timstr =
 		    date_sec2date_str((*rpt)->until,
 				      DATEFMT(conf.input_datefmt));
 		if (updatestring(win[STA].p, &timstr, 0, 1) !=
 		    GETSTRING_VALID) {
-			mem_free(timstr);
-			return;
+			goto cleanup;
 		}
 		if (strcmp(timstr, "0") == 0) {
 			newuntil = 0;
@@ -265,9 +258,10 @@ static void update_rept(struct rpt **rpt, const long start)
 		if (!parse_date
 		    (timstr, conf.input_datefmt, &newyear, &newmonth,
 		     &newday, ui_calendar_get_slctd_day())) {
-			snprintf(outstr, BUFSIZ, msg_fmts,
+			asprintf(&outstr, msg_fmts,
 				 DATEFMT_DESC(conf.input_datefmt));
 			status_mesg(msg_wrong_date, outstr);
+			mem_free(outstr);
 			wgetch(win[KEY].p);
 			continue;
 		}
@@ -283,10 +277,15 @@ static void update_rept(struct rpt **rpt, const long start)
 		wgetch(win[KEY].p);
 	}
 
-	mem_free(timstr);
 	(*rpt)->type = recur_char2def(newtype);
 	(*rpt)->freq = newfreq;
 	(*rpt)->until = newuntil;
+
+cleanup:
+	mem_free(msg_rpt_current);
+	mem_free(msg_rpt_asktype);
+	mem_free(freqstr);
+	mem_free(timstr);
 }
 
 /* Edit an already existing item. */
@@ -314,9 +313,11 @@ void ui_day_item_edit(void)
 			(_("Edit: "), choice_recur_evnt, 2)) {
 		case 1:
 			update_desc(&re->mesg);
+			io_set_modified();
 			break;
 		case 2:
 			update_rept(&re->rpt, re->day);
+			io_set_modified();
 			break;
 		default:
 			return;
@@ -325,6 +326,7 @@ void ui_day_item_edit(void)
 	case EVNT:
 		e = p->item.ev;
 		update_desc(&e->mesg);
+		io_set_modified();
 		break;
 	case RECUR_APPT:
 		ra = p->item.rapt;
@@ -340,23 +342,28 @@ void ui_day_item_edit(void)
 		case 1:
 			need_check_notify = 1;
 			update_start_time(&ra->start, &ra->dur, 1);
+			io_set_modified();
 			break;
 		case 2:
 			update_duration(&ra->start, &ra->dur);
+			io_set_modified();
 			break;
 		case 3:
 			if (notify_bar())
 				need_check_notify =
 				    notify_same_recur_item(ra);
 			update_desc(&ra->mesg);
+			io_set_modified();
 			break;
 		case 4:
 			need_check_notify = 1;
 			update_rept(&ra->rpt, ra->start);
+			io_set_modified();
 			break;
 		case 5:
 			need_check_notify = 1;
 			update_start_time(&ra->start, &ra->dur, 0);
+			io_set_modified();
 			break;
 		default:
 			return;
@@ -375,19 +382,23 @@ void ui_day_item_edit(void)
 		case 1:
 			need_check_notify = 1;
 			update_start_time(&a->start, &a->dur, 1);
+			io_set_modified();
 			break;
 		case 2:
 			update_duration(&a->start, &a->dur);
+			io_set_modified();
 			break;
 		case 3:
 			if (notify_bar())
 				need_check_notify =
 				    notify_same_item(a->start);
 			update_desc(&a->mesg);
+			io_set_modified();
 			break;
 		case 4:
 			need_check_notify = 1;
 			update_start_time(&a->start, &a->dur, 0);
+			io_set_modified();
 			break;
 		default:
 			return;
@@ -549,6 +560,7 @@ void ui_day_item_add(void)
 				  date2sec(*ui_calendar_get_slctd_day(), 0,
 					   0), 1);
 		}
+		io_set_modified();
 	}
 
 	ui_calendar_monthly_view_cache_set_invalid();
@@ -594,6 +606,7 @@ void ui_day_item_delete(unsigned reg)
 			break;
 		case 2:
 			day_item_erase_note(p);
+			io_set_modified();
 			return;
 		default:	/* User escaped */
 			return;
@@ -607,6 +620,7 @@ void ui_day_item_delete(unsigned reg)
 			break;
 		case 2:
 			day_item_add_exc(p, date);
+			io_set_modified();
 			return;
 		default:
 			return;
@@ -617,6 +631,7 @@ void ui_day_item_delete(unsigned reg)
 	p = day_cut_item(date, listbox_get_sel(&lb_apt));
 	day_cut[reg].type = p->type;
 	day_cut[reg].item = p->item;
+	io_set_modified();
 
 	ui_calendar_monthly_view_cache_set_invalid();
 }
@@ -634,7 +649,7 @@ void ui_day_item_repeat(void)
 	time_t t;
 	int year = 0, month = 0, day = 0;
 	struct date until_date;
-	char outstr[BUFSIZ];
+	char *outstr;
 	char user_input[BUFSIZ] = "";
 	const char *msg_rpt_prefix = _("Enter the repetition type:");
 	const char *msg_rpt_daily = _("(d)aily");
@@ -656,9 +671,8 @@ void ui_day_item_repeat(void)
 	const char *mesg_older =
 	    _("Sorry, the date you entered is older than the item start time.");
 
-	char msg_asktype[BUFSIZ];
-	snprintf(msg_asktype, BUFSIZ, "%s %s, %s, %s, %s",
-		 msg_rpt_prefix,
+	char *msg_asktype;
+	asprintf(&msg_asktype, "%s %s, %s, %s, %s", msg_rpt_prefix,
 		 msg_rpt_daily, msg_rpt_weekly, msg_rpt_monthly,
 		 msg_rpt_yearly);
 
@@ -669,14 +683,14 @@ void ui_day_item_repeat(void)
 	long until, date;
 
 	if (day_item_count(0) <= 0)
-		return;
+		goto cleanup;
 
 	item_nb = listbox_get_sel(&lb_apt);
 	p = day_get_item(item_nb);
 	if (p->type != APPT && p->type != EVNT) {
 		status_mesg(wrong_type_1, wrong_type_2);
 		wgetch(win[KEY].p);
-		return;
+		goto cleanup;
 	}
 
 	switch (status_ask_choice(msg_asktype, msg_type_choice, 4)) {
@@ -693,14 +707,14 @@ void ui_day_item_repeat(void)
 		type = RECUR_YEARLY;
 		break;
 	default:
-		return;
+		goto cleanup;
 	}
 
 	while (freq == 0) {
 		status_mesg(mesg_freq_1, "");
 		if (getstring(win[STA].p, user_input, BUFSIZ, 0, 1) !=
 		    GETSTRING_VALID)
-			return;
+			goto cleanup;
 		freq = atoi(user_input);
 		if (freq == 0) {
 			status_mesg(mesg_wrong_freq, wrong_type_2);
@@ -710,12 +724,13 @@ void ui_day_item_repeat(void)
 	}
 
 	for (;;) {
-		snprintf(outstr, BUFSIZ, mesg_until_1,
+		asprintf(&outstr, mesg_until_1,
 			 DATEFMT_DESC(conf.input_datefmt));
 		status_mesg(outstr, "");
+		mem_free(outstr);
 		if (getstring(win[STA].p, user_input, BUFSIZ, 0, 1) !=
 		    GETSTRING_VALID)
-			return;
+			goto cleanup;
 		if (strlen(user_input) == 1
 		    && strcmp(user_input, "0") == 0) {
 			until = 0;
@@ -736,9 +751,10 @@ void ui_day_item_repeat(void)
 			status_mesg(mesg_older, wrong_type_2);
 			wgetch(win[KEY].p);
 		} else {
-			snprintf(outstr, BUFSIZ, mesg_wrong_2,
+			asprintf(&outstr, mesg_wrong_2,
 				 DATEFMT_DESC(conf.input_datefmt));
 			status_mesg(mesg_wrong_1, outstr);
+			mem_free(outstr);
 			wgetch(win[KEY].p);
 		}
 	}
@@ -764,8 +780,12 @@ void ui_day_item_repeat(void)
 	p = day_cut_item(date, item_nb);
 	day_cut[REG_BLACK_HOLE].type = p->type;
 	day_cut[REG_BLACK_HOLE].item = p->item;
+	io_set_modified();
 
 	ui_calendar_monthly_view_cache_set_invalid();
+
+cleanup:
+	mem_free(msg_asktype);
 }
 
 /* Free the current cut item, if any. */
@@ -815,6 +835,7 @@ void ui_day_item_paste(unsigned reg)
 
 	day_item_fork(&day_cut[reg], &day);
 	day_paste_item(&day, ui_calendar_get_slctd_day_sec());
+	io_set_modified();
 
 	ui_calendar_monthly_view_cache_set_invalid();
 }
@@ -905,6 +926,7 @@ void ui_day_flag(void)
 
 	struct day_item *item = day_get_item(listbox_get_sel(&lb_apt));
 	day_item_switch_notify(item);
+	io_set_modified();
 }
 
 void ui_day_view_note(void)
@@ -923,4 +945,5 @@ void ui_day_edit_note(void)
 
 	struct day_item *item = day_get_item(listbox_get_sel(&lb_apt));
 	day_edit_note(item, conf.editor);
+	io_set_modified();
 }
