@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,6 +36,7 @@
 
 #include <time.h>
 #include <string.h>
+#include <strings.h>
 #include <stdlib.h>
 #include <limits.h>
 #include <unistd.h>
@@ -197,7 +198,7 @@ int status_ask_choice(const char *message, const char choice[],
 		      int nb_choice)
 {
 	int i, ch;
-	char tmp[BUFSIZ];
+	char *tmp;
 	/* "[4/2/f/t/w/.../Z] " */
 	char avail_choice[2 * nb_choice + 3];
 
@@ -205,9 +206,9 @@ int status_ask_choice(const char *message, const char choice[],
 	avail_choice[1] = '\0';
 
 	for (i = 1; i <= nb_choice; i++) {
-		snprintf(tmp, BUFSIZ, (i == nb_choice) ? "%c] " : "%c/",
-			 choice[i]);
+		asprintf(&tmp, (i == nb_choice) ? "%c] " : "%c/", choice[i]);
 		strcat(avail_choice, tmp);
+		mem_free(tmp);
 	}
 
 	status_mesg(message, avail_choice);
@@ -248,7 +249,7 @@ status_ask_simplechoice(const char *prefix, const char *choice[],
 			int nb_choice)
 {
 	int i;
-	char tmp[BUFSIZ];
+	char *tmp;
 	/* "(1) Choice1, (2) Choice2, (3) Choice3?" */
 	char choicestr[BUFSIZ];
 	/* Holds the characters to choose from ('1', '2', etc) */
@@ -261,10 +262,11 @@ status_ask_simplechoice(const char *prefix, const char *choice[],
 	strcpy(choicestr, prefix);
 
 	for (i = 0; i < nb_choice; i++) {
-		snprintf(tmp, BUFSIZ,
+		asprintf(&tmp,
 			 ((i + 1) == nb_choice) ? "(%d) %s?" : "(%d) %s, ",
 			 (i + 1), choice[i]);
 		strcat(choicestr, tmp);
+		mem_free(tmp);
 	}
 
 	return (status_ask_choice(choicestr, char_choice, nb_choice));
@@ -618,21 +620,16 @@ const char *get_tempdir(void)
  * Create a new unique file, and return a newly allocated string which contains
  * the random part of the file name.
  */
-char *new_tempfile(const char *prefix, int trailing_len)
+char *new_tempfile(const char *prefix)
 {
-	char fullname[BUFSIZ];
-	int prefix_len, fd;
+	char *fullname;
+	int fd;
 	FILE *file;
 
 	if (prefix == NULL)
 		return NULL;
 
-	prefix_len = strlen(prefix);
-	if (prefix_len + trailing_len >= BUFSIZ)
-		return NULL;
-	memcpy(fullname, prefix, prefix_len);
-	memset(fullname + prefix_len, 'X', trailing_len);
-	fullname[prefix_len + trailing_len] = '\0';
+	asprintf(&fullname, "%s.XXXXXX", prefix);
 	if ((fd = mkstemp(fullname)) == -1
 	    || (file = fdopen(fd, "w+")) == NULL) {
 		if (fd != -1) {
@@ -641,11 +638,39 @@ char *new_tempfile(const char *prefix, int trailing_len)
 		}
 		ERROR_MSG(_("temporary file \"%s\" could not be created"),
 			  fullname);
+
+		mem_free(fullname);
 		return NULL;
 	}
 	fclose(file);
 
-	return mem_strdup(fullname + prefix_len);
+	return fullname;
+}
+
+static void get_ymd(int *year, int *month, int *day, time_t t)
+{
+	struct tm tm;
+
+	localtime_r(&t, &tm);
+	*day = tm.tm_mday;
+	*month = tm.tm_mon + 1;
+	*year = tm.tm_year + 1900;
+}
+
+static void get_weekday_ymd(int *year, int *month, int *day, int weekday)
+{
+	time_t t = get_today();
+	struct tm tm;
+	int delta;
+
+	localtime_r(&t, &tm);
+	delta = weekday - tm.tm_wday;
+	t = date_sec_change(t, 0, delta > 0 ? delta : 7);
+
+	localtime_r(&t, &tm);
+	*day = tm.tm_mday;
+	*month = tm.tm_mon + 1;
+	*year = tm.tm_year + 1900;
 }
 
 /*
@@ -653,10 +678,8 @@ char *new_tempfile(const char *prefix, int trailing_len)
  */
 int check_date(unsigned year, unsigned month, unsigned day)
 {
-	return (year >= 1902 && year <= 2037 && month >= 1 && month <= 12
-		&& day >= 1 && day <= days[month - 1] + (month == 2
-							 && ISLEAP(year)) ?
-		1 : 0);
+	return (year >= 1902 && month >= 1 && month <= 12 && day >= 1 &&
+		day <= days[month - 1] + (month == 2 && ISLEAP(year)) ? 1 : 0);
 }
 
 /*
@@ -683,6 +706,48 @@ parse_date(const char *date_string, enum datefmt datefmt, int *year,
 	if (!date_string)
 		return 0;
 
+	if (!strcasecmp(date_string, "today")) {
+		get_ymd(year, month, day, get_today());
+		return 1;
+	} else if (!strcasecmp(date_string, "yesterday")) {
+		get_ymd(year, month, day, date_sec_change(get_today(), 0, -1));
+		return 1;
+	} else if (!strcasecmp(date_string, "tomorrow")) {
+		get_ymd(year, month, day, date_sec_change(get_today(), 0, 1));
+		return 1;
+	} else if (!strcasecmp(date_string, "now")) {
+		get_ymd(year, month, day, now());
+		return 1;
+	} else if (!strcasecmp(date_string, "sunday") ||
+		   !strcasecmp(date_string, "sun")) {
+		get_weekday_ymd(year, month, day, 0);
+		return 1;
+	} else if (!strcasecmp(date_string, "monday") ||
+		   !strcasecmp(date_string, "mon")) {
+		get_weekday_ymd(year, month, day, 1);
+		return 1;
+	} else if (!strcasecmp(date_string, "tuesday") ||
+		   !strcasecmp(date_string, "tue")) {
+		get_weekday_ymd(year, month, day, 2);
+		return 1;
+	} else if (!strcasecmp(date_string, "wednesday") ||
+		   !strcasecmp(date_string, "wed")) {
+		get_weekday_ymd(year, month, day, 3);
+		return 1;
+	} else if (!strcasecmp(date_string, "thursday") ||
+		   !strcasecmp(date_string, "thu")) {
+		get_weekday_ymd(year, month, day, 4);
+		return 1;
+	} else if (!strcasecmp(date_string, "friday") ||
+		   !strcasecmp(date_string, "fri")) {
+		get_weekday_ymd(year, month, day, 5);
+		return 1;
+	} else if (!strcasecmp(date_string, "saturday") ||
+		   !strcasecmp(date_string, "sat")) {
+		get_weekday_ymd(year, month, day, 6);
+		return 1;
+	}
+
 	/* parse string into in[], read up to three integers */
 	for (p = date_string; *p; p++) {
 		if (*p == sep) {
@@ -984,8 +1049,7 @@ shell_exec(int *pfdin, int *pfdout, const char *path,
 	narg[1] = "-c";
 
 	if (argc > 1) {
-		arg0 = mem_malloc(strlen(path) + 6);
-		sprintf(arg0, "%s \"$@\"", path);
+		asprintf(&arg0, "%s \"$@\"", path);
 		narg[2] = arg0;
 
 		for (i = 0; i < argc; i++)
@@ -1049,7 +1113,7 @@ void press_any_key(void)
  */
 static void print_notefile(FILE * out, const char *filename, int nbtab)
 {
-	char path_to_notefile[BUFSIZ];
+	char *path_to_notefile;
 	FILE *notefile;
 	char linestarter[BUFSIZ];
 	char buffer[BUFSIZ];
@@ -1064,8 +1128,9 @@ static void print_notefile(FILE * out, const char *filename, int nbtab)
 		linestarter[0] = '\0';
 	}
 
-	snprintf(path_to_notefile, BUFSIZ, "%s/%s", path_notes, filename);
+	asprintf(&path_to_notefile, "%s/%s", path_notes, filename);
 	notefile = fopen(path_to_notefile, "r");
+	mem_free(path_to_notefile);
 	if (notefile) {
 		while (fgets(buffer, BUFSIZ, notefile) != 0) {
 			if (printlinestarter) {
@@ -1223,13 +1288,14 @@ static void print_date(long date, long day, const char *extformat)
 	if (!strcmp(extformat, "epoch")) {
 		printf("%ld", date);
 	} else {
+		time_t day_end = date_sec_change(day, 0, 1);
 		time_t t = date;
 		struct tm lt;
 
 		localtime_r((time_t *) & t, &lt);
 
 		if (extformat[0] == '\0' || !strcmp(extformat, "default")) {
-			if (date >= day && date <= day + DAYINSEC)
+			if (date >= day && date <= day_end)
 				strftime(buf, BUFSIZ, "%H:%M", &lt);
 			else
 				strftime(buf, BUFSIZ, "..:..", &lt);
@@ -1473,3 +1539,41 @@ void print_todo(const char *format, struct todo *todo)
 		}
 	}
 }
+
+#define VASPRINTF_INITIAL_BUFSIZE 128
+
+int
+vasprintf(char **str, const char *format, va_list ap)
+{
+	va_list ap2;
+	int n;
+
+	va_copy(ap2, ap);
+
+	*str = mem_malloc(VASPRINTF_INITIAL_BUFSIZE);
+	n = vsnprintf(*str, VASPRINTF_INITIAL_BUFSIZE, format, ap);
+	if (n >= VASPRINTF_INITIAL_BUFSIZE) {
+		*str = mem_realloc(*str, 1, n + 1);
+		n = vsnprintf(*str, n + 1, format, ap2);
+	}
+
+	if (n < 0) {
+		mem_free(*str);
+		*str = NULL;
+	}
+
+	return n;
+}
+
+int
+asprintf(char **str, const char *format, ...)
+{
+	va_list	ap;
+	int n;
+
+	va_start(ap, format);
+	n = vasprintf(str, format, ap);
+	va_end(ap);
+
+	return n;
+}
