@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -61,6 +61,9 @@ enum ui_mode ui_mode = UI_CMDLINE;
 /* Don't save anything if this is set. */
 int read_only = 0;
 
+/* Applications can trigger a reload by sending SIGUSR1. */
+int want_reload = 0;
+
 /* Strings describing each input date format. */
 const char *datefmt_str[DATE_FORMATS];
 
@@ -126,7 +129,7 @@ struct dmon_conf dmon;
  */
 void vars_init(void)
 {
-	const char *ed, *pg;
+	const char *ed, *pg, *mt;
 
 	/* Variables for user configuration */
 	conf.confirm_quit = 1;
@@ -159,6 +162,11 @@ void vars_init(void)
 		pg = DEFAULT_PAGER;
 	conf.pager = pg;
 
+	mt = getenv("MERGETOOL");
+	if (mt == NULL || mt[0] == '\0')
+		mt = DEFAULT_MERGETOOL;
+	conf.mergetool = mt;
+
 	wins_set_layout(1);
 
 	ui_calendar_set_first_day_of_week(MONDAY);
