@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2013 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -544,8 +544,10 @@ void wins_update(int flags)
 {
 	wins_update_border(flags);
 	wins_update_panels(flags);
-	if (flags & FLAG_STA)
+	if (flags & FLAG_STA) {
+		wins_update_bindings();
 		wins_status_bar();
+	}
 	if ((flags & FLAG_NOT) && notify_bar())
 		notify_update_bar();
 	wmove(win[STA].p, 0, 0);
@@ -600,9 +602,8 @@ void wins_unprepare_external(void)
  * While inside interactive mode, launch the external command cmd on the given
  * file.
  */
-void wins_launch_external(const char *file, const char *cmd)
+void wins_launch_external(const char *arg[])
 {
-	const char *arg[] = { cmd, file, NULL };
 	int pid;
 
 	wins_prepare_external();
@@ -611,121 +612,99 @@ void wins_launch_external(const char *file, const char *cmd)
 	wins_unprepare_external();
 }
 
-#define NB_CAL_CMDS    27	/* number of commands while in cal view */
-#define NB_APP_CMDS    32	/* same thing while in appointment view */
-#define NB_TOD_CMDS    31	/* same thing while in todo view */
-
+static int *bindings;
+static int bindings_size;
 static unsigned status_page;
 
-/*
- * Draws the status bar.
- * To add a keybinding, insert a new binding_t item, add it in the *binding
- * table, and update the NB_CAL_CMDS, NB_APP_CMDS or NB_TOD_CMDS defines,
- * depending on which panel the added keybind is assigned to.
- */
-void wins_status_bar(void)
+/* Sets the current set of key bindings to display in the status bar. */
+void wins_set_bindings(int *new_bindings, int size)
 {
-	struct binding help = { _("Help"), KEY_GENERIC_HELP };
-	struct binding quit = { _("Quit"), KEY_GENERIC_QUIT };
-	struct binding save = { _("Save"), KEY_GENERIC_SAVE };
-	struct binding copy = { _("Copy"), KEY_GENERIC_COPY };
-	struct binding paste = { _("Paste"), KEY_GENERIC_PASTE };
-	struct binding chgvu = { _("Chg Win"), KEY_GENERIC_CHANGE_VIEW };
-	struct binding import = { _("Import"), KEY_GENERIC_IMPORT };
-	struct binding export = { _("Export"), KEY_GENERIC_EXPORT };
-	struct binding togo = { _("Go to"), KEY_GENERIC_GOTO };
-	struct binding conf = { _("Config"), KEY_GENERIC_CONFIG_MENU };
-	struct binding draw = { _("Redraw"), KEY_GENERIC_REDRAW };
-	struct binding appt = { _("Add Appt"), KEY_GENERIC_ADD_APPT };
-	struct binding todo = { _("Add Todo"), KEY_GENERIC_ADD_TODO };
-	struct binding gpday = { _("-1 Day"), KEY_GENERIC_PREV_DAY };
-	struct binding gnday = { _("+1 Day"), KEY_GENERIC_NEXT_DAY };
-	struct binding gpweek = { _("-1 Week"), KEY_GENERIC_PREV_WEEK };
-	struct binding gnweek = { _("+1 Week"), KEY_GENERIC_NEXT_WEEK };
-	struct binding gpmonth = { _("-1 Month"), KEY_GENERIC_PREV_MONTH };
-	struct binding gnmonth = { _("+1 Month"), KEY_GENERIC_NEXT_MONTH };
-	struct binding gpyear = { _("-1 Year"), KEY_GENERIC_PREV_YEAR };
-	struct binding gnyear = { _("+1 Year"), KEY_GENERIC_NEXT_YEAR };
-	struct binding today = { _("Today"), KEY_GENERIC_GOTO_TODAY };
-	struct binding nview = { _("Nxt View"), KEY_GENERIC_SCROLL_DOWN };
-	struct binding pview = { _("Prv View"), KEY_GENERIC_SCROLL_UP };
-	struct binding cmd = { _("Command"), KEY_GENERIC_CMD };
-	struct binding up = { _("Up"), KEY_MOVE_UP };
-	struct binding down = { _("Down"), KEY_MOVE_DOWN };
-	struct binding left = { _("Left"), KEY_MOVE_LEFT };
-	struct binding right = { _("Right"), KEY_MOVE_RIGHT };
-	struct binding weekb = { _("beg Week"), KEY_START_OF_WEEK };
-	struct binding weeke = { _("end Week"), KEY_END_OF_WEEK };
-	struct binding add = { _("Add Item"), KEY_ADD_ITEM };
-	struct binding del = { _("Del Item"), KEY_DEL_ITEM };
-	struct binding edit = { _("Edit Itm"), KEY_EDIT_ITEM };
-	struct binding view = { _("View"), KEY_VIEW_ITEM };
-	struct binding pipe = { _("Pipe"), KEY_PIPE_ITEM };
-	struct binding flag = { _("Flag Itm"), KEY_FLAG_ITEM };
-	struct binding rept = { _("Repeat"), KEY_REPEAT_ITEM };
-	struct binding enote = { _("EditNote"), KEY_EDIT_NOTE };
-	struct binding vnote = { _("ViewNote"), KEY_VIEW_NOTE };
-	struct binding rprio = { _("Prio.+"), KEY_RAISE_PRIORITY };
-	struct binding lprio = { _("Prio.-"), KEY_LOWER_PRIORITY };
-	struct binding othr = { _("OtherCmd"), KEY_GENERIC_OTHER_CMD };
+	bindings = new_bindings;
+	bindings_size = size;
+}
 
-	struct binding *bindings_cal[] = {
-		&help, &quit, &save, &chgvu, &nview, &pview, &up, &down,
-		    &left, &right,
-		&togo, &import, &export, &weekb, &weeke, &appt, &todo,
-		    &gpday, &gnday,
-		&gpweek, &gnweek, &gpmonth, &gnmonth, &gpyear, &gnyear,
-		    &draw, &today,
-		&conf, &cmd
+/*
+ * Obtains the set of key bindings to display for the active panel.
+ *
+ * To add a key binding, insert a new binding_t item and add it to the binding
+ * table.
+ */
+void wins_update_bindings(void)
+{
+	static int bindings_cal[] = {
+		KEY_GENERIC_HELP, KEY_GENERIC_QUIT, KEY_GENERIC_SAVE,
+		KEY_GENERIC_RELOAD, KEY_GENERIC_CHANGE_VIEW,
+		KEY_GENERIC_SCROLL_DOWN, KEY_GENERIC_SCROLL_UP, KEY_MOVE_UP,
+		KEY_MOVE_DOWN, KEY_MOVE_LEFT, KEY_MOVE_RIGHT, KEY_GENERIC_GOTO,
+		KEY_GENERIC_IMPORT, KEY_GENERIC_EXPORT, KEY_START_OF_WEEK,
+		KEY_END_OF_WEEK, KEY_GENERIC_ADD_APPT, KEY_GENERIC_ADD_TODO,
+		KEY_GENERIC_PREV_DAY, KEY_GENERIC_NEXT_DAY,
+		KEY_GENERIC_PREV_WEEK, KEY_GENERIC_NEXT_WEEK,
+		KEY_GENERIC_PREV_MONTH, KEY_GENERIC_NEXT_MONTH,
+		KEY_GENERIC_PREV_YEAR, KEY_GENERIC_NEXT_YEAR,
+		KEY_GENERIC_REDRAW, KEY_GENERIC_GOTO_TODAY,
+		KEY_GENERIC_CONFIG_MENU, KEY_GENERIC_CMD
 	};
 
-	struct binding *bindings_apoint[] = {
-		&help, &quit, &save, &chgvu, &import, &export, &add, &del,
-		    &edit, &view,
-		&pipe, &draw, &rept, &flag, &enote, &vnote, &up, &down,
-		    &gpday, &gnday,
-		&gpweek, &gnweek, &gpmonth, &gnmonth, &gpyear, &gnyear,
-		    &togo, &today,
-		&conf, &appt, &todo, &copy, &paste, &cmd
+	static int bindings_apoint[] = {
+		KEY_GENERIC_HELP, KEY_GENERIC_QUIT, KEY_GENERIC_SAVE,
+		KEY_GENERIC_RELOAD, KEY_GENERIC_CHANGE_VIEW,
+		KEY_GENERIC_IMPORT, KEY_GENERIC_EXPORT, KEY_ADD_ITEM,
+		KEY_DEL_ITEM, KEY_EDIT_ITEM, KEY_VIEW_ITEM, KEY_PIPE_ITEM,
+		KEY_GENERIC_REDRAW, KEY_REPEAT_ITEM, KEY_FLAG_ITEM,
+		KEY_EDIT_NOTE, KEY_VIEW_NOTE, KEY_MOVE_UP, KEY_MOVE_DOWN,
+		KEY_GENERIC_PREV_DAY, KEY_GENERIC_NEXT_DAY,
+		KEY_GENERIC_PREV_WEEK, KEY_GENERIC_NEXT_WEEK,
+		KEY_GENERIC_PREV_MONTH, KEY_GENERIC_NEXT_MONTH,
+		KEY_GENERIC_PREV_YEAR, KEY_GENERIC_NEXT_YEAR, KEY_GENERIC_GOTO,
+		KEY_GENERIC_GOTO_TODAY, KEY_GENERIC_CONFIG_MENU,
+		KEY_GENERIC_ADD_APPT, KEY_GENERIC_ADD_TODO, KEY_GENERIC_COPY,
+		KEY_GENERIC_PASTE, KEY_GENERIC_CMD
 	};
 
-	struct binding *bindings_todo[] = {
-		&help, &quit, &save, &chgvu, &import, &export, &add, &del,
-		    &edit, &view,
-		&pipe, &flag, &rprio, &lprio, &enote, &vnote, &up, &down,
-		    &gpday, &gnday,
-		&gpweek, &gnweek, &gpmonth, &gnmonth, &gpyear, &gnyear,
-		    &togo, &today,
-		&conf, &appt, &todo, &draw, &cmd
+	static int bindings_todo[] = {
+		KEY_GENERIC_HELP, KEY_GENERIC_QUIT, KEY_GENERIC_SAVE,
+		KEY_GENERIC_RELOAD, KEY_GENERIC_CHANGE_VIEW,
+		KEY_GENERIC_IMPORT, KEY_GENERIC_EXPORT, KEY_ADD_ITEM,
+		KEY_DEL_ITEM, KEY_EDIT_ITEM, KEY_VIEW_ITEM, KEY_PIPE_ITEM,
+		KEY_FLAG_ITEM, KEY_RAISE_PRIORITY, KEY_LOWER_PRIORITY,
+		KEY_EDIT_NOTE, KEY_VIEW_NOTE, KEY_MOVE_UP, KEY_MOVE_DOWN,
+		KEY_GENERIC_PREV_DAY, KEY_GENERIC_NEXT_DAY,
+		KEY_GENERIC_PREV_WEEK, KEY_GENERIC_NEXT_WEEK,
+		KEY_GENERIC_PREV_MONTH, KEY_GENERIC_NEXT_MONTH,
+		KEY_GENERIC_PREV_YEAR, KEY_GENERIC_NEXT_YEAR, KEY_GENERIC_GOTO,
+		KEY_GENERIC_GOTO_TODAY, KEY_GENERIC_CONFIG_MENU,
+		KEY_GENERIC_ADD_APPT, KEY_GENERIC_ADD_TODO, KEY_GENERIC_REDRAW,
+		KEY_GENERIC_CMD
 	};
 
 	enum win active_panel = wins_slctd();
 
-	struct binding **bindings;
-	int bindings_size;
-
 	switch (active_panel) {
 	case CAL:
-		bindings = bindings_cal;
-		bindings_size = ARRAY_SIZE(bindings_cal);
+		wins_set_bindings(bindings_cal, ARRAY_SIZE(bindings_cal));
 		break;
 	case APP:
-		bindings = bindings_apoint;
-		bindings_size = ARRAY_SIZE(bindings_apoint);
+		wins_set_bindings(bindings_apoint,
+				ARRAY_SIZE(bindings_apoint));
 		break;
 	case TOD:
-		bindings = bindings_todo;
-		bindings_size = ARRAY_SIZE(bindings_todo);
+		wins_set_bindings(bindings_todo, ARRAY_SIZE(bindings_todo));
 		break;
 	default:
 		EXIT(_("unknown panel"));
 		/* NOTREACHED */
 	}
+}
+
+/* Draws the status bar. */
+void wins_status_bar(void)
+{
+	int page_base = (KEYS_CMDS_PER_LINE * 2 - 1) * (status_page - 1);
+	int page_size = KEYS_CMDS_PER_LINE * 2;
 
 	keys_display_bindings_bar(win[STA].p, bindings, bindings_size,
-				  (KEYS_CMDS_PER_LINE * 2 -
-				   1) * (status_page - 1),
-				  KEYS_CMDS_PER_LINE * 2, &othr);
+				  page_base, page_size);
 }
 
 /* Erase status bar. */
@@ -737,23 +716,12 @@ void wins_erase_status_bar(void)
 /* Update the status bar page number to display other commands. */
 void wins_other_status_page(int panel)
 {
-	int nb_item, max_page;
+	int max_page = bindings_size / (KEYS_CMDS_PER_LINE * 2 - 1) + 1;
+
+	/* There is no "OtherCmd" on the last page. */
+	if (bindings_size % (KEYS_CMDS_PER_LINE * 2 - 1) == 1)
+		max_page--;
 
-	switch (panel) {
-	case CAL:
-		nb_item = NB_CAL_CMDS;
-		break;
-	case APP:
-		nb_item = NB_APP_CMDS;
-		break;
-	case TOD:
-		nb_item = NB_TOD_CMDS;
-		break;
-	default:
-		EXIT(_("unknown panel"));
-		/* NOTREACHED */
-	}
-	max_page = nb_item / (KEYS_CMDS_PER_LINE * 2 - 1) + 1;
 	status_page = (status_page % max_page) + 1;
 }
 
@@ -762,7 +730,3 @@ void wins_reset_status_page(void)
 {
 	status_page = 1;
 }
-
-#undef NB_CAL_CMDS
-#undef NB_APP_CMDS
-#undef NB_TOD_CMDS
